@@ -1,7 +1,7 @@
 /*
  *   Canon SELPHY ES/CP series CUPS backend -- libusb-1.0 version
  *
- *   (c) 2007-2018 Solomon Peachy <pizza@shaftnet.org>
+ *   (c) 2007-2020 Solomon Peachy <pizza@shaftnet.org>
  *
  *   The latest version of this program can be found at:
  *
@@ -26,16 +26,6 @@
  *
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <unistd.h>
-
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <signal.h>
-
 #define BACKEND canonselphy_backend
 
 #include "backend_common.h"
@@ -79,7 +69,7 @@
 
 struct printer_data {
 	int  type;  /* P_??? */
-	char *model; /* eg "SELPHY ES1" */
+	const char *model; /* eg "SELPHY ES1" */
 	uint16_t init_length;
 	uint16_t  foot_length;
 	int16_t init_readback[READBACK_LEN];
@@ -94,10 +84,10 @@ struct printer_data {
 	int8_t  paper_code_offset; /* Offset in readback for paper type */
 	int8_t  paper_code_offset2; /* Offset in readback for paper type (2nd) */
 	uint8_t (*error_detect)(uint8_t *rdbuf);
-	char    *(*pgcode_names)(uint8_t *rdbuf, struct printer_data *printer);
+	const char    *(*pgcode_names)(uint8_t *rdbuf, struct printer_data *printer, int *numtype);
 };
 
-static char *generic_pgcode_names(uint8_t *rdbuf, struct printer_data *printer)
+static const char *generic_pgcode_names(uint8_t *rdbuf, struct printer_data *printer, int *numtype)
 {
 	uint8_t pgcode = 0, pgcode2 = 0;
 
@@ -106,6 +96,8 @@ static char *generic_pgcode_names(uint8_t *rdbuf, struct printer_data *printer)
 	if (printer->paper_code_offset2 != -1)
 		pgcode2 = rdbuf[printer->paper_code_offset2];
 
+	*numtype = pgcode & 0xf;
+
 	switch(pgcode & 0xf) {
 	case 0x01: return "P";
 	case 0x02: return "L";
@@ -133,7 +125,7 @@ static uint8_t es1_error_detect(uint8_t *rdbuf)
 		return 1;
 	}
 
-	return 0;
+	return CUPS_BACKEND_OK;
 }
 
 static uint8_t es2_error_detect(uint8_t *rdbuf)
@@ -157,7 +149,7 @@ static uint8_t es2_error_detect(uint8_t *rdbuf)
 		return 1;
 	}
 
-	return 0;
+	return CUPS_BACKEND_OK;
 }
 
 static uint8_t es3_error_detect(uint8_t *rdbuf)
@@ -191,14 +183,14 @@ static uint8_t es3_error_detect(uint8_t *rdbuf)
 		return 1;
 	}
 
-	return 0;
+	return CUPS_BACKEND_OK;
 }
 
 static uint8_t es40_error_detect(uint8_t *rdbuf)
 {
 	/* ES40 */
 	if (!rdbuf[3])
-		return 0;
+		return CUPS_BACKEND_OK;
 
 	if (rdbuf[3] == 0x01)
 		ERROR("Generic communication error\n");
@@ -233,21 +225,22 @@ static uint8_t cp790_error_detect(uint8_t *rdbuf)
 		return 1;
 	}
 
-	return 0;
+	return CUPS_BACKEND_OK;
 }
 
-static char *cp10_pgcode_names(uint8_t *rdbuf, struct printer_data *printer)
+static const char *cp10_pgcode_names(uint8_t *rdbuf, struct printer_data *printer, int *numtype)
 {
 	UNUSED(rdbuf);
 	UNUSED(printer);
 
+	*numtype = 3;
 	return "C";   /* Printer only supports one media type */
 }
 
 static uint8_t cp10_error_detect(uint8_t *rdbuf)
 {
 	if (!rdbuf[2])
-		return 0;
+		return CUPS_BACKEND_OK;
 
 	if (rdbuf[2] == 0x80)
 		ERROR("No ribbon loaded\n");
@@ -263,7 +256,7 @@ static uint8_t cp10_error_detect(uint8_t *rdbuf)
 static uint8_t cpxxx_error_detect(uint8_t *rdbuf)
 {
 	if (!rdbuf[2])
-		return 0;
+		return CUPS_BACKEND_OK;
 
 	if (rdbuf[2] == 0x01)
 		ERROR("Paper feed problem!\n");
@@ -466,9 +459,9 @@ enum {
 	S_FINISHED,
 };
 
-static int fancy_memcmp(const uint8_t *buf_a, const int16_t *buf_b, uint len)
+static int fancy_memcmp(const uint8_t *buf_a, const int16_t *buf_b, uint16_t len)
 {
-	uint i;
+	uint16_t i;
 
 	for (i = 0 ; i < len ; i++) {
 		if (buf_b[i] == -1)
@@ -478,7 +471,7 @@ static int fancy_memcmp(const uint8_t *buf_a, const int16_t *buf_b, uint len)
 		else if (buf_a[i] < buf_b[i])
 			return -1;
 	}
-	return 0;
+	return CUPS_BACKEND_OK;
 }
 
 static int parse_printjob(uint8_t *buffer, uint8_t *bw_mode, uint32_t *plane_len)
@@ -551,10 +544,7 @@ struct canonselphy_printjob {
 };
 
 struct canonselphy_ctx {
-	struct libusb_device_handle *dev;
-	uint8_t endp_up;
-	uint8_t endp_down;
-	int type;
+	struct dyesub_connection *conn;
 
 	struct printer_data *printer;
 	struct marker marker;
@@ -568,18 +558,18 @@ static int canonselphy_get_status(struct canonselphy_ctx *ctx)
 	int ret, num;
 
 	/* Read in the printer status, twice. */
-	ret = read_data(ctx->dev, ctx->endp_up,
-			(uint8_t*) rdbuf, READBACK_LEN, &num);
+	ret = read_data(ctx->conn,
+			 (uint8_t*) rdbuf, READBACK_LEN, &num);
 	if (ret < 0)
 		return CUPS_BACKEND_FAILED;
 
-	ret = read_data(ctx->dev, ctx->endp_up,
-			(uint8_t*) rdbuf, READBACK_LEN, &num);
+	ret = read_data(ctx->conn,
+			 (uint8_t*) rdbuf, READBACK_LEN, &num);
 
 	if (ret < 0)
 		return CUPS_BACKEND_FAILED;
 
-	INFO("Media type: %s\n", ctx->printer->pgcode_names? ctx->printer->pgcode_names(rdbuf, ctx->printer) : "Unknown");
+	INFO("Media type: %s\n", ctx->printer->pgcode_names? ctx->printer->pgcode_names(rdbuf, ctx->printer, &ret) : "Unknown");
 	ctx->printer->error_detect(rdbuf);
 
 	return CUPS_BACKEND_OK;
@@ -592,8 +582,8 @@ static int canonselphy_send_reset(struct canonselphy_ctx *ctx)
 			       0x00, 0x00, 0x00, 0x00 };
 	int ret;
 
-	if ((ret = send_data(ctx->dev, ctx->endp_down,
-			     rstcmd, sizeof(rstcmd))))
+	if ((ret = send_data(ctx->conn,
+			      rstcmd, sizeof(rstcmd))))
 		return CUPS_BACKEND_FAILED;
 
 	return CUPS_BACKEND_OK;
@@ -614,28 +604,21 @@ static void *canonselphy_init(void)
 	return ctx;
 }
 
-extern struct dyesub_backend canonselphy_backend;
-
-static int canonselphy_attach(void *vctx, struct libusb_device_handle *dev, int type,
-			      uint8_t endp_up, uint8_t endp_down, uint8_t jobid)
+static int canonselphy_attach(void *vctx, struct dyesub_connection *conn, uint8_t jobid)
 {
 	struct canonselphy_ctx *ctx = vctx;
 	int i, num;
 	uint8_t rdbuf[READBACK_LEN];
-
 	UNUSED(jobid);
 
-	ctx->dev = dev;
-	ctx->endp_up = endp_up;
-	ctx->endp_down = endp_down;
-	ctx->type = type;
+	ctx->conn = conn;
 
-	if (ctx->type == P_CP900) {
-		ctx->type = P_CP_XXX;
+	if (ctx->conn->type == P_CP900) {
+		ctx->conn->type = P_CP_XXX;
 		ctx->cp900 = 1;
 	}
 	for (i = 0 ; selphy_printers[i].type != -1; i++) {
-		if (selphy_printers[i].type == ctx->type) {
+		if (selphy_printers[i].type == ctx->conn->type) {
 			ctx->printer = &selphy_printers[i];
 		}
 	}
@@ -646,29 +629,31 @@ static int canonselphy_attach(void *vctx, struct libusb_device_handle *dev, int
 
 	/* Fill out marker structure */
 	ctx->marker.color = "#00FFFF#FF00FF#FFFF00";
-	ctx->marker.levelmax = -1; /* Unknown */
+	ctx->marker.levelmax = CUPS_MARKER_UNAVAILABLE;
 
 	if (test_mode < TEST_MODE_NOATTACH) {
 		/* Read printer status. Twice. */
-		i = read_data(ctx->dev, ctx->endp_up,
+		i = read_data(ctx->conn,
 			      rdbuf, READBACK_LEN, &num);
 		if (i < 0)
 			return CUPS_BACKEND_FAILED;
 
-		i = read_data(ctx->dev, ctx->endp_up,
+		i = read_data(ctx->conn,
 			      rdbuf, READBACK_LEN, &num);
 		if (i < 0)
 			return CUPS_BACKEND_FAILED;
 
 		if (ctx->printer->error_detect(rdbuf))
-			ctx->marker.levelnow = 0;  /* Out of media */
+			ctx->marker.levelnow = 0;
 		else
-			ctx->marker.levelnow = -3; /* Unknown but OK */
-		ctx->marker.name = ctx->printer->pgcode_names? ctx->printer->pgcode_names(rdbuf, ctx->printer) : "Unknown";
+			ctx->marker.levelnow = CUPS_MARKER_UNKNOWN_OK;
+
+		ctx->marker.name = ctx->printer->pgcode_names? ctx->printer->pgcode_names(rdbuf, ctx->printer, &ctx->marker.numtype) : "Unknown";
 	} else {
 		// XXX handle MEDIA_CODE at some point.
 		// we don't do any error checking here.
 		ctx->marker.name = "Unknown";
+		ctx->marker.numtype = -1;
 	}
 
 	return CUPS_BACKEND_OK;
@@ -688,7 +673,7 @@ static void canonselphy_cleanup_job(const void *vjob) {
 	if (job->footer)
 		free(job->footer);
 
-	free((void*)job);
+	free((void*)vjob);
 }
 
 static int canonselphy_read_parse(void *vctx, const void **vjob, int data_fd, int copies)
@@ -751,14 +736,14 @@ static int canonselphy_read_parse(void *vctx, const void **vjob, int data_fd, in
 	printer_type = parse_printjob(rdbuf, &job->bw_mode, &job->plane_len);
 	/* Special cases for some models */
 	if (printer_type == P_ES40_CP790) {
-		if (ctx->type == P_CP790)
+		if (ctx->conn->type == P_CP790)
 			printer_type = P_CP790;
 		else
 			printer_type = P_ES40;
 	}
 
-	if (printer_type != ctx->type) {
-		ERROR("Printer/Job mismatch (%d/%d/%d)\n", ctx->type, ctx->printer->type, printer_type);
+	if (printer_type != ctx->conn->type) {
+		ERROR("Printer/Job mismatch (%d/%d/%d)\n", ctx->conn->type, ctx->printer->type, printer_type);
 		canonselphy_cleanup_job(job);
 		return CUPS_BACKEND_CANCEL;
 	}
@@ -861,7 +846,7 @@ static int canonselphy_main_loop(void *vctx, const void *vjob) {
 	copies = job->copies;
 
 	/* Read in the printer status to clear last state */
-	ret = read_data(ctx->dev, ctx->endp_up,
+	ret = read_data(ctx->conn,
 			rdbuf, READBACK_LEN, &num);
 
 	if (ret < 0)
@@ -874,7 +859,7 @@ top:
 	}
 
 	/* Read in the printer status */
-	ret = read_data(ctx->dev, ctx->endp_up,
+	ret = read_data(ctx->conn,
 			rdbuf, READBACK_LEN, &num);
 	if (ret < 0)
 		return CUPS_BACKEND_FAILED;
@@ -889,7 +874,7 @@ top:
 		dump_markers(&ctx->marker, 1, 0);
 		if (ctx->printer->clear_error_len)
 			/* Try to clear error state */
-			if ((ret = send_data(ctx->dev, ctx->endp_down, ctx->printer->clear_error, ctx->printer->clear_error_len)))
+			if ((ret = send_data(ctx->conn, ctx->printer->clear_error, ctx->printer->clear_error_len)))
 				return CUPS_BACKEND_FAILED;
 		return CUPS_BACKEND_HOLD;
 	}
@@ -901,7 +886,7 @@ top:
 	}
 	last_state = state;
 
-	fflush(stderr);
+	fflush(logger);
 
 	switch(state) {
 	case S_IDLE:
@@ -911,7 +896,7 @@ top:
 
 		/* Make sure paper/ribbon is correct */
 		if (job->paper_code != -1) {
-			if (ctx->type == P_CP_XXX) {
+			if (ctx->conn->type == P_CP_XXX) {
 				uint8_t pc = rdbuf[ctx->printer->paper_code_offset];
 				if (((pc >> 4) & 0xf) != (job->paper_code & 0x0f)) {
 
@@ -942,7 +927,7 @@ top:
 					return CUPS_BACKEND_HOLD;  /* Hold this job, don't stop queue */
 				}
 			}
-		} else if (ctx->type == P_CP790) {
+		} else if (ctx->conn->type == P_CP790) {
 			uint8_t ribbon = rdbuf[4] >> 4;
 			uint8_t paper = rdbuf[5];
 
@@ -967,7 +952,7 @@ top:
 	case S_PRINTER_READY:
 		INFO("Printing started; Sending init sequence\n");
 		/* Send printer init */
-		if ((ret = send_data(ctx->dev, ctx->endp_down, job->header, ctx->printer->init_length)))
+		if ((ret = send_data(ctx->conn, job->header, ctx->printer->init_length)))
 			return CUPS_BACKEND_FAILED;
 
 		state = S_PRINTER_INIT_SENT;
@@ -983,7 +968,7 @@ top:
 		else
 			INFO("Sending YELLOW plane\n");
 
-		if ((ret = send_data(ctx->dev, ctx->endp_down, job->plane_y, job->plane_len)))
+		if ((ret = send_data(ctx->conn, job->plane_y, job->plane_len)))
 			return CUPS_BACKEND_FAILED;
 
 		state = S_PRINTER_Y_SENT;
@@ -999,7 +984,7 @@ top:
 	case S_PRINTER_READY_M:
 		INFO("Sending MAGENTA plane\n");
 
-		if ((ret = send_data(ctx->dev, ctx->endp_down, job->plane_m, job->plane_len)))
+		if ((ret = send_data(ctx->conn, job->plane_m, job->plane_len)))
 			return CUPS_BACKEND_FAILED;
 
 		state = S_PRINTER_M_SENT;
@@ -1012,7 +997,7 @@ top:
 	case S_PRINTER_READY_C:
 		INFO("Sending CYAN plane\n");
 
-		if ((ret = send_data(ctx->dev, ctx->endp_down, job->plane_c, job->plane_len)))
+		if ((ret = send_data(ctx->conn, job->plane_c, job->plane_len)))
 			return CUPS_BACKEND_FAILED;
 
 		state = S_PRINTER_C_SENT;
@@ -1029,7 +1014,7 @@ top:
 		uint32_t empty = 0;
 
 		INFO("Sending CP900 Footer\n");
-		if ((ret = send_data(ctx->dev, ctx->endp_down,
+		if ((ret = send_data(ctx->conn,
 				     (uint8_t*)&empty, sizeof(empty))))
 			return CUPS_BACKEND_FAILED;
 
@@ -1040,7 +1025,7 @@ top:
 		if (ctx->printer->foot_length) {
 			INFO("Cleaning up\n");
 
-			if ((ret = send_data(ctx->dev, ctx->endp_down, job->footer, ctx->printer->foot_length)))
+			if ((ret = send_data(ctx->conn, job->footer, ctx->printer->foot_length)))
 				return CUPS_BACKEND_FAILED;
 		}
 		state = S_FINISHED;
@@ -1088,7 +1073,7 @@ static int canonselphy_cmdline_arg(void *vctx, int argc, char **argv)
 		if (j) return j;
 	}
 
-	return 0;
+	return CUPS_BACKEND_OK;
 }
 
 static void canonselphy_cmdline(void)
@@ -1104,12 +1089,12 @@ static int canonselphy_query_markers(void *vctx, struct marker **markers, int *c
 	int ret, num;
 
 	/* Read in the printer status, twice. */
-	ret = read_data(ctx->dev, ctx->endp_up,
+	ret = read_data(ctx->conn,
 			(uint8_t*) rdbuf, READBACK_LEN, &num);
 	if (ret < 0)
 		return CUPS_BACKEND_FAILED;
 
-	ret = read_data(ctx->dev, ctx->endp_up,
+	ret = read_data(ctx->conn,
 			(uint8_t*) rdbuf, READBACK_LEN, &num);
 
 	if (ret < 0)
@@ -1118,7 +1103,7 @@ static int canonselphy_query_markers(void *vctx, struct marker **markers, int *c
 	if (ctx->printer->error_detect(rdbuf))
 		ctx->marker.levelnow = 0;
 	else
-		ctx->marker.levelnow = -3;
+		ctx->marker.levelnow = CUPS_MARKER_UNKNOWN_OK;
 
 	*markers = &ctx->marker;
 	*count = 1;
@@ -1128,14 +1113,6 @@ static int canonselphy_query_markers(void *vctx, struct marker **markers, int *c
 
 static const char *canonselphy_prefixes[] = {
 	"canonselphy", // Family name
-	"canon-cp10", "canon-cp100", "canon-cp200", "canon-cp220",
-	"canon-cp300", "canon-cp330", "canon-cp400", "canon-cp500",
-	"canon-cp510", "canon-cp520", "canon-cp530", "canon-cp600",
-	"canon-cp710", "canon-cp720", "canon-cp730", "canon-cp740",
-	"canon-cp750", "canon-cp760", "canon-cp770", "canon-cp780",
-	"canon-cp790", "canon-cp800", "canon-cp810", "canon-cp900",
-	"canon-es1", "canon-es2", "canon-es20", "canon-es3",
-	"canon-es30", "canon-es40",
 	// backwards compatibility
 	"selphycp10", "selphycp100", "selphycp200", "selphycp220",
 	"selphycp300", "selphycp330", "selphycp400", "selphycp500",
@@ -1148,9 +1125,9 @@ static const char *canonselphy_prefixes[] = {
 	NULL
 };
 
-struct dyesub_backend canonselphy_backend = {
+const struct dyesub_backend canonselphy_backend = {
 	.name = "Canon SELPHY CP/ES (legacy)",
-	.version = "0.104",
+	.version = "0.106",
 	.uri_prefixes = canonselphy_prefixes,
 	.cmdline_usage = canonselphy_cmdline,
 	.cmdline_arg = canonselphy_cmdline_arg,
@@ -1165,7 +1142,7 @@ struct dyesub_backend canonselphy_backend = {
 		{ USB_VID_CANON, USB_PID_CANON_CP100, P_CP_XXX, NULL, "canon-cp100"},
 		{ USB_VID_CANON, USB_PID_CANON_CP200, P_CP_XXX, NULL, "canon-cp200"},
 		{ USB_VID_CANON, USB_PID_CANON_CP220, P_CP_XXX, NULL, "canon-cp220"},
-		{ USB_VID_CANON, USB_PID_CANON_CP300, P_CP_XXX, NULL, "selpyhcp300"},
+		{ USB_VID_CANON, USB_PID_CANON_CP300, P_CP_XXX, NULL, "canon-cp300"},
 		{ USB_VID_CANON, USB_PID_CANON_CP330, P_CP_XXX, NULL, "canon-cp330"},
 		{ USB_VID_CANON, USB_PID_CANON_CP400, P_CP_XXX, NULL, "canon-cp400"},
 		{ USB_VID_CANON, USB_PID_CANON_CP500, P_CP_XXX, NULL, "canon-cp500"},
