@@ -1,7 +1,7 @@
 /*
  *   Kodak Professional 1400/805 CUPS backend -- libusb-1.0 version
  *
- *   (c) 2013-2019 Solomon Peachy <pizza@shaftnet.org>
+ *   (c) 2013-2020 Solomon Peachy <pizza@shaftnet.org>
  *
  *   The latest version of this program can be found at:
  *
@@ -26,16 +26,6 @@
  *
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <unistd.h>
-
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <signal.h>
-
 #define BACKEND kodak1400_backend
 
 #include "backend_common.h"
@@ -86,10 +76,7 @@ struct kodak1400_printjob {
 };
 
 struct kodak1400_ctx {
-	struct libusb_device_handle *dev;
-	uint8_t endp_up;
-	uint8_t endp_down;
-	int type;
+	struct dyesub_connection *conn;
 
 	struct marker marker;
 };
@@ -109,7 +96,7 @@ static int send_plane(struct kodak1400_ctx *ctx,
 		cmdbuf[2] = 0x00;
 		cmdbuf[3] = 0x50;
 
-		if ((ret = send_data(ctx->dev, ctx->endp_down,
+		if ((ret = send_data(ctx->conn,
 				     cmdbuf, CMDBUF_LEN)))
 			return ret;
 	}
@@ -121,20 +108,20 @@ static int send_plane(struct kodak1400_ctx *ctx,
 	cmdbuf[3] = planeno;
 
 	if (planedata) {
-		temp16 = htons(job->hdr.columns);
+		temp16 = be16_to_cpu(job->hdr.columns);
 		memcpy(cmdbuf+7, &temp16, 2);
-		temp16 = htons(job->hdr.rows);
+		temp16 = be16_to_cpu(job->hdr.rows);
 		memcpy(cmdbuf+9, &temp16, 2);
 	}
 
-	if ((ret = send_data(ctx->dev, ctx->endp_down,
+	if ((ret = send_data(ctx->conn,
 			     cmdbuf, CMDBUF_LEN)))
 		return ret;
 
 	if (planedata) {
 		int i;
 		for (i = 0 ; i < job->hdr.rows ; i++) {
-			if ((ret = send_data(ctx->dev, ctx->endp_down,
+			if ((ret = send_data(ctx->conn,
 					     planedata + i * job->hdr.columns,
 					     job->hdr.columns)))
 				return ret;
@@ -147,20 +134,16 @@ static int send_plane(struct kodak1400_ctx *ctx,
 	cmdbuf[2] = 0x01;
 	cmdbuf[3] = 0x50;
 
-	if ((ret = send_data(ctx->dev, ctx->endp_down,
+	if ((ret = send_data(ctx->conn,
 			     cmdbuf, CMDBUF_LEN)))
 		return ret;
 
-	return 0;
+	return CUPS_BACKEND_OK;
 }
 
 #define TONE_CURVE_SIZE 1552
 static int kodak1400_set_tonecurve(struct kodak1400_ctx *ctx, char *fname)
 {
-	libusb_device_handle *dev = ctx->dev;
-	uint8_t endp_down = ctx->endp_down;
-	uint8_t endp_up = ctx->endp_up;
-
 	uint8_t cmdbuf[8];
 	uint8_t respbuf[64];
 	int ret = 0, num = 0;
@@ -191,13 +174,13 @@ static int kodak1400_set_tonecurve(struct kodak1400_ctx *ctx, char *fname)
 	memset(cmdbuf, 0, sizeof(cmdbuf));
 	cmdbuf[0] = 0x1b;
 	cmdbuf[1] = 0xa2;
-	if ((ret = send_data(dev, endp_down,
+	if ((ret = send_data(ctx->conn,
 			     cmdbuf, 2))) {
 		ret = -3;
 		goto done;
 	}
 
-	ret = read_data(dev, endp_up,
+	ret = read_data(ctx->conn,
 			respbuf, sizeof(respbuf), &num);
 
 	if (ret < 0)
@@ -221,17 +204,17 @@ static int kodak1400_set_tonecurve(struct kodak1400_ctx *ctx, char *fname)
 	cmdbuf[3] = 0x03;
 	cmdbuf[4] = 0x06;
 	cmdbuf[5] = 0x10;   /* 06 10 == TONE_CURVE_SIZE */
-	if ((ret = send_data(dev, endp_down,
+	if ((ret = send_data(ctx->conn,
 			     cmdbuf, 6)))
 		goto done;
 
 	/* Send the payload over */
-	if ((ret = send_data(dev, endp_down,
+	if ((ret = send_data(ctx->conn,
 			     (uint8_t *) data, TONE_CURVE_SIZE)))
 		goto done;
 
 	/* get the response */
-	ret = read_data(dev, endp_up,
+	ret = read_data(ctx->conn,
 			respbuf, sizeof(respbuf), &num);
 
 	if (ret < 0)
@@ -258,7 +241,7 @@ static void kodak1400_cmdline(void)
 	DEBUG("\t\t[ -C filename ]  # Set tone curve\n");
 }
 
-int kodak1400_cmdline_arg(void *vctx, int argc, char **argv)
+static int kodak1400_cmdline_arg(void *vctx, int argc, char **argv)
 {
 	struct kodak1400_ctx *ctx = vctx;
 	int i, j = 0;
@@ -279,7 +262,7 @@ int kodak1400_cmdline_arg(void *vctx, int argc, char **argv)
 		if (j) return j;
 	}
 
-	return 0;
+	return CUPS_BACKEND_OK;
 }
 
 static void *kodak1400_init(void)
@@ -294,22 +277,20 @@ static void *kodak1400_init(void)
 	return ctx;
 }
 
-static int kodak1400_attach(void *vctx, struct libusb_device_handle *dev, int type,
-			    uint8_t endp_up, uint8_t endp_down, uint8_t jobid)
+static int kodak1400_attach(void *vctx, struct dyesub_connection *conn,
+			    uint8_t jobid)
 {
 	struct kodak1400_ctx *ctx = vctx;
 
 	UNUSED(jobid);
 
-	ctx->dev = dev;
-	ctx->endp_up = endp_up;
-	ctx->endp_down = endp_down;
-	ctx->type = type;
+	ctx->conn = conn;
 
 	ctx->marker.color = "#00FFFF#FF00FF#FFFF00";
 	ctx->marker.name = "Unknown";
-	ctx->marker.levelmax = -1;
-	ctx->marker.levelnow = -2;
+	ctx->marker.numtype = -1;
+	ctx->marker.levelmax = CUPS_MARKER_UNAVAILABLE;
+	ctx->marker.levelnow = CUPS_MARKER_UNKNOWN;
 
 	return CUPS_BACKEND_OK;
 }
@@ -442,12 +423,12 @@ top:
 	cmdbuf[0] = 0x1b;
 	cmdbuf[1] = 0x72;
 
-	if ((ret = send_data(ctx->dev, ctx->endp_down,
+	if ((ret = send_data(ctx->conn,
 			    cmdbuf, CMDBUF_LEN)))
 		return CUPS_BACKEND_FAILED;
 
 	/* Read in the printer status */
-	ret = read_data(ctx->dev, ctx->endp_up,
+	ret = read_data(ctx->conn,
 			rdbuf, READBACK_LEN, &num);
 
 	if (ret < 0)
@@ -466,7 +447,7 @@ top:
 		return CUPS_BACKEND_STOP;  // HOLD/CANCEL/FAILED?  XXXX parse error!
 	}
 
-	fflush(stderr);
+	fflush(logger);
 
 	switch (state) {
 	case S_IDLE:
@@ -476,7 +457,7 @@ top:
 		memset(cmdbuf, 0, CMDBUF_LEN);
 		cmdbuf[0] = 0x1b;
 
-		if ((ret = send_data(ctx->dev, ctx->endp_down,
+		if ((ret = send_data(ctx->conn,
 				     cmdbuf, CMDBUF_LEN)))
 			return CUPS_BACKEND_FAILED;
 
@@ -490,7 +471,7 @@ top:
 		temp16 = be16_to_cpu(job->hdr.rows);
 		memcpy(cmdbuf+5, &temp16, 2);
 
-		if ((ret = send_data(ctx->dev, ctx->endp_down,
+		if ((ret = send_data(ctx->conn,
 				    cmdbuf, CMDBUF_LEN)))
 			return CUPS_BACKEND_FAILED;
 
@@ -500,7 +481,7 @@ top:
 		cmdbuf[1] = 0x59;
 		cmdbuf[2] = job->hdr.matte; // ???
 
-		if ((ret = send_data(ctx->dev, ctx->endp_down,
+		if ((ret = send_data(ctx->conn,
 				    cmdbuf, CMDBUF_LEN)))
 			return CUPS_BACKEND_FAILED;
 
@@ -510,7 +491,7 @@ top:
 		cmdbuf[1] = 0x60;
 		cmdbuf[2] = job->hdr.laminate;
 
-		if (send_data(ctx->dev, ctx->endp_down,
+		if (send_data(ctx->conn,
 			     cmdbuf, CMDBUF_LEN))
 			return CUPS_BACKEND_FAILED;
 
@@ -520,7 +501,7 @@ top:
 		cmdbuf[1] = 0x62;
 		cmdbuf[2] = job->hdr.lam_strength;
 
-		if ((ret = send_data(ctx->dev, ctx->endp_down,
+		if ((ret = send_data(ctx->conn,
 				    cmdbuf, CMDBUF_LEN)))
 			return CUPS_BACKEND_FAILED;
 
@@ -530,7 +511,7 @@ top:
 		cmdbuf[1] = 0x61;
 		cmdbuf[2] = job->hdr.unk1; // ???
 
-		if ((ret = send_data(ctx->dev, ctx->endp_down,
+		if ((ret = send_data(ctx->conn,
 				    cmdbuf, CMDBUF_LEN)))
 			return CUPS_BACKEND_FAILED;
 
@@ -589,7 +570,7 @@ top:
 		cmdbuf[2] = 0x00;
 		cmdbuf[3] = 0x50;
 
-		if ((ret = send_data(ctx->dev, ctx->endp_down,
+		if ((ret = send_data(ctx->conn,
 				    cmdbuf, CMDBUF_LEN)))
 			return CUPS_BACKEND_FAILED;
 
@@ -636,17 +617,14 @@ static int kodak1400_query_markers(void *vctx, struct marker **markers, int *cou
 
 static const char *kodak1400_prefixes[] = {
 	"kodak1400", // Family driver, do NOT nuke!
-	"kodak-1400", "kodak-805", "mitsubishi-3020d", "mitsubishi-3020da",
 	// backwards compatibility
-	"kodak805", "mitsu3020d", "mitsu3020da",
-	// Extras.
-	"mitsubishi-3020dae", "mitsubishi-3020de", "mitsubishi-3020du",
+	"kodak805",
 	NULL,
 };
 
-struct dyesub_backend kodak1400_backend = {
+const struct dyesub_backend kodak1400_backend = {
 	.name = "Kodak 1400/805",
-	.version = "0.40",
+	.version = "0.41",
 	.uri_prefixes = kodak1400_prefixes,
 	.cmdline_usage = kodak1400_cmdline,
 	.cmdline_arg = kodak1400_cmdline_arg,
@@ -660,7 +638,10 @@ struct dyesub_backend kodak1400_backend = {
 		{ USB_VID_KODAK, USB_PID_KODAK_1400, P_KODAK_1400_805, "Kodak", "kodak-1400"},
 		{ USB_VID_KODAK, USB_PID_KODAK_805, P_KODAK_1400_805, "Kodak", "kodak-805"},
 		{ USB_VID_MITSU, USB_PID_MITSU_3020D, P_KODAK_1400_805, NULL, "mitsubishi-3020d"},
+		{ USB_VID_MITSU, USB_PID_MITSU_3020D, P_KODAK_1400_805, NULL, "mitsubishi-3020du"}, /* Duplicate */
+		{ USB_VID_MITSU, USB_PID_MITSU_3020D, P_KODAK_1400_805, NULL, "mitsubishi-3020de"}, /* Duplicate */
 		{ USB_VID_MITSU, USB_PID_MITSU_3020DA, P_KODAK_1400_805, NULL, "mitsubishi-3020da" },
+		{ USB_VID_MITSU, USB_PID_MITSU_3020DA, P_KODAK_1400_805, NULL, "mitsubishi-3020dae" }, /* Duplicate */
 		{ 0, 0, 0, NULL, NULL}
 	}
 };
