@@ -1,7 +1,7 @@
 /*
  *   Kodak 605 Photo Printer CUPS backend -- libusb-1.0 version
  *
- *   (c) 2013-2019 Solomon Peachy <pizza@shaftnet.org>
+ *   (c) 2013-2020 Solomon Peachy <pizza@shaftnet.org>
  *
  *   The latest version of this program can be found at:
  *
@@ -26,16 +26,6 @@
  *
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <unistd.h>
-
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <signal.h>
-
 #define BACKEND kodak605_backend
 
 #include "backend_common.h"
@@ -47,12 +37,6 @@
 #define USB_PID_KODAK_7010  0x4037
 #define USB_PID_KODAK_7015  0x4038
 
-/* List of confirmed commands */
-//#define SINFONIA_CMD_GETSTATUS  0x0001
-//#define SINFONIA_CMD_MEDIAINFO  0x0002
-//#define SINFONIA_CMD_PRINTJOB   0x4001
-//#define SINFONIA_CMD_UPDATE     0xC004
-
 /* Media structure */
 struct kodak605_media_list {
 	struct sinfonia_status_hdr hdr;
@@ -62,7 +46,7 @@ struct kodak605_media_list {
 	struct sinfonia_mediainfo_item entries[];
 } __attribute__((packed));
 
-#define MAX_MEDIA_LEN 128
+#define MAX_MEDIA_LEN (sizeof(struct kodak605_media_list) + sizeof(struct sinfonia_mediainfo_item) * 10)
 
 /* Status response */
 struct kodak605_status {
@@ -78,22 +62,80 @@ struct kodak605_status {
 	uint16_t b1_remain;
 	uint16_t b1_complete;
 	uint16_t b1_total;
-/*@45*/	uint8_t  b1_sts;    /* See BANK_STATUS_* */
-	uint8_t  b2_id;     /* jobid */
+	uint8_t  b1_sts;    /* See BANK_STATUS_* */
+/*@46*/	uint8_t  b2_id;     /* jobid */
 	uint16_t b2_remain;
 	uint16_t b2_complete;
 	uint16_t b2_total;
-/*@53*/	uint8_t  b2_sts;    /* see BANK_STATUS_* */
-/*@54*/	uint8_t  id;        /* current job id ( 00/01/02 seen ) */
-/*@55*/ uint16_t remain;    /* in current job */
-/*@57*/	uint16_t complete;  /* in current job */
-/*@59*/	uint16_t total;     /* in current job */
-/*@61*/	uint8_t  null_2[9]; /* 00 00 00 00 00 00 00 00 00 */
-/*@70*/	uint8_t  unk_12[6]; /* 01 00 00 00 00 00 (605) 01 01 01 01 00 00 (EK7000) */
-/*@76*/	uint8_t  unk_13[1]; // EK7000-series only?
+	uint8_t  b2_sts;    /* see BANK_STATUS_* */
+/*@54*/	uint8_t  b3_id;     /* jobid */
+	uint16_t b3_remain;
+	uint16_t b3_complete;
+	uint16_t b3_total;
+	uint8_t  b3_sts;    /* see BANK_STATUS_* */
+/*@62*/	uint8_t  b4_id;     /* jobid */
+	uint16_t b4_remain;
+	uint16_t b4_complete;
+	uint16_t b4_total;
+	uint8_t  b4_sts;    /* see BANK_STATUS_* */
+/*@70*/	uint8_t  unk[4];  /* EK605 has 01/00/00/00, EK7000 has 01/01/01/01 */
+/*@74*/	uint8_t  null_2[2]; /* 00 00 */
+/*@76*/	uint8_t  null_3[1]; /* EK7000 only */
 } __attribute__((packed));
 
-#define CMDBUF_LEN 4
+static const struct sinfonia_param ek7000_params[] =
+{
+	{ 0x01, "Unknown_01" }, // 00000001
+	{ 0x11, "Unknown_11" }, // 00000001
+	{ 0x12, "Matte Gloss" }, // 00000069
+	{ 0x13, "Matte Degloss Black" }, // 000000c3
+	{ 0x14, "Matte Degloss White" }, // 000000cd
+	{ 0x21, "Exit Speed With Sorter 4x6" }, // 000003e8
+	{ 0x22, "Exit Speed With Sorter 8x6" }, // 0000041a
+	{ 0x23, "Exit Speed With Backprinting" }, // 00000152
+	{ 0x24, "Exit Speed Without PPAC 4x6" }, // 0000044c
+	{ 0x25, "Exit Speed Without PPAC 8x6" }, // 0000044c
+
+	{ 0x2f, "Unknown_2f" }, // 00000320
+	{ 0x41, "Unknown_41" }, // 0000006d
+	{ 0x42, "Unknown_42" }, // 00000051
+	{ 0x43, "Unknown_43" }, // 0000003b
+	{ 0x44, "Unknown_44" }, // 00000082
+	{ 0x45, "Unknown_45" }, // 00000000
+	{ 0x46, "Unknown_46" }, // 00000000
+	{ 0x47, "Unknown_47" }, // 00000028
+	{ 0x48, "Unknown_48" }, // 00000002
+	{ 0x81, "Unknown_81" }, // ffffffff
+
+	{ 0x82, "Unknown_82" }, // fffffffe
+	{ 0x83, "Unknown_83" }, // ffffffee
+	{ 0x84, "Unknown_84" }, // 00000001
+	{ 0x91, "Unknown_91" }, // 0000006c
+	{ 0x92, "Unknown_92" }, // 00000077
+	{ 0x93, "Unknown_93" }, // 00000067
+	{ 0x94, "Unknown_94" }, // 00000076
+	{ 0xa0, "Unknown_a0" }, // 00000005
+	{ 0xa1, "Unknown_a1" }, // 00000000
+	{ 0xa2, "Unknown_a2" }, // 00000010
+
+	{ 0xa3, "Unknown_a3" }, // 0000003b
+	{ 0xa4, "Unknown_a4" }, // 0000003b
+	{ 0xa5, "Thermal Protect Lamination" }, // 0000003e
+	{ 0xa6, "Unknown_a6" }, // 00000001
+	{ 0xa7, "Unknown_a7" }, // 00000014
+	{ 0xa8, "Unknown_a8" }, // 00000001
+	{ 0xa9, "Unknown_a9" }, // ffffffff
+	{ 0xc1, "Unknown_c1" }, // 00000002
+	{ 0xc2, "Unknown_c2" }, // 000000c8
+	{ 0xc3, "Unknown_c3" }, // 000000c8
+
+	{ 0xc4, "Unknown_c4" }, // 00000200
+	{ 0xf1, "Unknown_f1" }, // 00000068
+	{ 0xf2, "Unknown_f2" }, // 00000068
+	{ 0xf3, "Unknown_f3" }, // 00000094
+	{ 0xf4, "Unknown_f4" }, // 00000068
+};
+#define ek7000_params_num (sizeof(ek7000_params) / sizeof(struct sinfonia_param))
 
 /* Private data structure */
 struct kodak605_ctx {
@@ -103,6 +145,9 @@ struct kodak605_ctx {
 
 	struct kodak605_media_list *media;
 
+	char serial[32];
+	char fwver[32];
+
 	struct marker marker;
 };
 
@@ -282,30 +327,6 @@ static const char *error_codes(uint8_t major, uint8_t minor)
 	}
 }
 
-static int kodak605_get_media(struct kodak605_ctx *ctx, struct kodak605_media_list *media)
-{
-	struct sinfonia_cmd_hdr cmd;
-
-	int i, ret, num = 0;
-
-	cmd.cmd = cpu_to_le16(SINFONIA_CMD_MEDIAINFO);
-	cmd.len = cpu_to_le16(0);
-
-	if ((ret = sinfonia_docmd(&ctx->dev,
-				  (uint8_t*)&cmd, sizeof(cmd),
-				  (uint8_t*)media, MAX_MEDIA_LEN,
-				  &num))) {
-		return ret;
-	}
-
-	for (i = 0 ; i < media->count; i++) {
-		media->entries[i].rows = le16_to_cpu(media->entries[i].rows);
-		media->entries[i].columns = le16_to_cpu(media->entries[i].columns);
-	}
-
-	return 0;
-}
-
 static int kodak605_get_status(struct kodak605_ctx *ctx, struct kodak605_status *sts)
 {
 	struct sinfonia_cmd_hdr cmd;
@@ -317,11 +338,11 @@ static int kodak605_get_status(struct kodak605_ctx *ctx, struct kodak605_status
 
 	if ((ret = sinfonia_docmd(&ctx->dev,
 				  (uint8_t*)&cmd, sizeof(cmd),
-				  (uint8_t*)sts, sizeof(*sts), &num)) < 0) {
+				  (uint8_t*)sts, sizeof(*sts), &num))) {
 		return ret;
 	}
 
-	return 0;
+	return CUPS_BACKEND_OK;
 }
 
 static void *kodak605_init(void)
@@ -338,17 +359,18 @@ static void *kodak605_init(void)
 	return ctx;
 }
 
-static int kodak605_attach(void *vctx, struct libusb_device_handle *dev, int type,
-			   uint8_t endp_up, uint8_t endp_down, uint8_t jobid)
+static int kodak605_attach(void *vctx, struct dyesub_connection *conn, uint8_t jobid)
 {
 	struct kodak605_ctx *ctx = vctx;
 
-	ctx->dev.dev = dev;
-	ctx->dev.endp_up = endp_up;
-	ctx->dev.endp_down = endp_down;
-	ctx->dev.type = type;
+	ctx->dev.conn = conn;
 	ctx->dev.error_codes = &error_codes;
 
+	if (ctx->dev.conn->type != P_KODAK_605) {
+		ctx->dev.params = ek7000_params;
+		ctx->dev.params_count = ek7000_params_num;
+	}
+
 	/* Make sure jobid is sane */
 	ctx->jobid = jobid & 0x7f;
 	if (!ctx->jobid)
@@ -356,10 +378,10 @@ static int kodak605_attach(void *vctx, struct libusb_device_handle *dev, int typ
 
 	if (test_mode < TEST_MODE_NOATTACH) {
 		/* Query media info */
-		if (kodak605_get_media(ctx, ctx->media)) {
-			ERROR("Can't query media\n");
-			return CUPS_BACKEND_FAILED;
-		}
+		int ret = sinfonia_query_media(&ctx->dev,
+					       ctx->media);
+		if (ret)
+			return ret;
 	} else {
 		int media_code = KODAK6_MEDIA_6TR2;
 		if (getenv("MEDIA_CODE"))
@@ -370,8 +392,9 @@ static int kodak605_attach(void *vctx, struct libusb_device_handle *dev, int typ
 
 	ctx->marker.color = "#00FFFF#FF00FF#FFFF00";
 	ctx->marker.name = kodak6_mediatypes(ctx->media->type);
+	ctx->marker.numtype = ctx->media->type;
 	ctx->marker.levelmax = 100; /* Ie percentage */
-	ctx->marker.levelnow = -2;
+	ctx->marker.levelnow = CUPS_MARKER_UNKNOWN;
 
 	return CUPS_BACKEND_OK;
 }
@@ -399,7 +422,7 @@ static int kodak605_read_parse(void *vctx, const void **vjob, int data_fd, int c
 		return ret;
 	}
 
-	/* Printer handles generating copies.. */
+	/* Printer handles generating copies.  Use larger of our options */
 	if (le16_to_cpu(job->jp.copies) < (uint16_t)copies)
 		job->jp.copies = cpu_to_le16(copies);
 
@@ -414,6 +437,7 @@ static int kodak605_main_loop(void *vctx, const void *vjob) {
 	struct kodak605_status sts;
 
 	int num, ret;
+	int offset = 0;
 
 	const struct sinfonia_printjob *job = vjob;
 
@@ -433,7 +457,14 @@ static int kodak605_main_loop(void *vctx, const void *vjob) {
 		return CUPS_BACKEND_HOLD;
 	}
 
-	INFO("Waiting for printer idle\n");
+	/* > 4x6 jobs need two banks */
+	int banks_needed;
+	if (job->jp.rows > 1240)
+		banks_needed = 2;
+	else
+		banks_needed = 1;
+
+	INFO("Waiting for printer idle (%d banks needed)\n", banks_needed);
 
 	while(1) {
 		if ((ret = kodak605_get_status(ctx, &sts)))
@@ -458,22 +489,70 @@ static int kodak605_main_loop(void *vctx, const void *vjob) {
 		/* Make sure we're not colliding with an existing
 		   jobid */
 		while (ctx->jobid == sts.b1_id ||
-		       ctx->jobid == sts.b2_id) {
+		       ctx->jobid == sts.b2_id ||
+		       ctx->jobid == sts.b3_id ||
+		       ctx->jobid == sts.b4_id) {
 			ctx->jobid++;
 			ctx->jobid &= 0x7f;
 			if (!ctx->jobid)
 				ctx->jobid++;
 		}
 
-		/* Wait for a free buffer */
-		if (sts.b1_sts == BANK_STATUS_FREE ||
-		    sts.b2_sts == BANK_STATUS_FREE) {
+		int banks_free = 0;
+		if (sts.b1_sts == BANK_STATUS_FREE)
+			banks_free++;
+		if (sts.b2_sts == BANK_STATUS_FREE)
+			banks_free++;
+		if (sts.b3_sts == BANK_STATUS_FREE)
+			banks_free++;
+		if (sts.b4_sts == BANK_STATUS_FREE)
+			banks_free++;
+
+		/* Do we have enough free buffers? */
+		if (banks_free >= banks_needed) {
 			break;
 		}
 
 		sleep(1);
 	}
 
+	/* Send backprint */
+	if ((job->jp.ext_flags & EXT_FLAG_BACKPRINT) && offset == 0) {
+		struct kodak701x_backprint bp;
+		INFO("Sending backprint text..\n");
+		bp.hdr.cmd = cpu_to_le16(SINFONIA_CMD_BACKPRINT);
+		bp.hdr.len = cpu_to_le16(sizeof(bp) - sizeof(bp.hdr));
+
+		/* Line 1 */
+		bp.unk_0 = job->databuf[offset + 0];
+		memset(bp.null, 0, sizeof(bp.null));
+		bp.unk_1 = job->databuf[offset + 1];
+		memcpy(bp.text, &job->databuf[offset + 2], sizeof(bp.text));
+
+		if ((ret = sinfonia_docmd(&ctx->dev,
+					  (uint8_t*)&bp, sizeof(bp),
+					  (uint8_t*)&sts.hdr, sizeof(sts.hdr),
+					  &num))) {
+			return ret;
+		}
+		offset += 44;
+
+		/* Line 2 */
+		bp.unk_0 = job->databuf[offset + 0];
+		memset(bp.null, 0, sizeof(bp.null));
+		bp.unk_1 = job->databuf[offset + 1];
+		memcpy(bp.text, &job->databuf[offset + 2], sizeof(bp.text));
+
+		if ((ret = sinfonia_docmd(&ctx->dev,
+					  (uint8_t*)&bp, sizeof(bp),
+					  (uint8_t*)&sts.hdr, sizeof(sts.hdr),
+					  &num))) {
+			return ret;
+		}
+		offset += 44;
+		// XXX sanity check backprint parameters..
+	}
+
 	/* Send print job */
 	struct sinfonia_printcmd10_hdr hdr;
 
@@ -494,7 +573,7 @@ retry_print:
 	if ((ret = sinfonia_docmd(&ctx->dev,
 				  (uint8_t*)&hdr, sizeof(hdr),
 				  (uint8_t*)&sts.hdr, sizeof(sts.hdr),
-				  &num)) < 0) {
+				  &num))) {
 		return ret;
 	}
 
@@ -519,8 +598,8 @@ retry_print:
 	}
 
 	INFO("Sending image data\n");
-	if ((ret = send_data(ctx->dev.dev, ctx->dev.endp_down,
-			     job->databuf, job->datalen)))
+	if ((ret = send_data(ctx->dev.conn,
+			     job->databuf + offset, job->datalen - offset)))
 		return CUPS_BACKEND_FAILED;
 
 	INFO("Waiting for printer to acknowledge completion\n");
@@ -552,6 +631,10 @@ retry_print:
 			break;
 		if (sts.b2_id == ctx->jobid && sts.b2_complete == sts.b2_total)
 			break;
+		if (sts.b3_id == ctx->jobid && sts.b3_complete == sts.b3_total)
+			break;
+		if (sts.b4_id == ctx->jobid && sts.b4_complete == sts.b4_total)
+			break;
 
 		if (sts.hdr.status == STATUS_READY)
 			break;
@@ -575,32 +658,25 @@ static void kodak605_dump_status(struct kodak605_ctx *ctx, struct kodak605_statu
 	     sts->hdr.printer_major, sts->hdr.printer_minor,
 	     error_codes(sts->hdr.printer_major, sts->hdr.printer_minor));
 
-	INFO("Bank 1: %s Job %03u @ %03u/%03u\n",
+	INFO("Bank 1: %s - Job %03u @ %03u/%03u\n",
 	     sinfonia_bank_statuses(sts->b1_sts), sts->b1_id,
 	     le16_to_cpu(sts->b1_complete), le16_to_cpu(sts->b1_total));
-	INFO("Bank 2: %s Job %03u @ %03u/%03u\n",
+	INFO("Bank 2: %s - Job %03u @ %03u/%03u\n",
 	     sinfonia_bank_statuses(sts->b2_sts), sts->b2_id,
 	     le16_to_cpu(sts->b2_complete), le16_to_cpu(sts->b2_total));
+	INFO("Bank 3: %s - Job %03u @ %03u/%03u\n",
+	     sinfonia_bank_statuses(sts->b3_sts), sts->b3_id,
+	     le16_to_cpu(sts->b3_complete), le16_to_cpu(sts->b3_total));
+	INFO("Bank 4: %s - Job %03u @ %03u/%03u\n",
+	     sinfonia_bank_statuses(sts->b4_sts), sts->b4_id,
+	     le16_to_cpu(sts->b4_complete), le16_to_cpu(sts->b4_total));
 
 	INFO("Lifetime prints   : %u\n", le32_to_cpu(sts->ctr_life));
 	INFO("Cutter actuations : %u\n", le32_to_cpu(sts->ctr_cut));
 	INFO("Head prints       : %u\n", le32_to_cpu(sts->ctr_head));
 	INFO("Media prints      : %u\n", le32_to_cpu(sts->ctr_media));
 	{
-		int max;
-
-		switch(ctx->media->type) {
-		case KODAK6_MEDIA_6R:
-		case KODAK6_MEDIA_6TR2:
-			max = 375;
-			break;
-		case KODAK7_MEDIA_6R:
-			max = 570;
-			break;
-		default:
-			max = 0;
-			break;
-		}
+		int max = kodak6_mediamax(ctx->media->type);
 
 		if (max) {
 			INFO("\t  Remaining     : %u\n", max - le32_to_cpu(sts->ctr_media));
@@ -634,6 +710,7 @@ static void kodak605_dump_mediainfo(struct kodak605_media_list *media)
 
 static void kodak605_cmdline(void)
 {
+	DEBUG("\t\t[ -b 0|1 ]       # Disable/Enable control panel\n");
 	DEBUG("\t\t[ -c filename ]  # Get user/NV tone curve\n");
 	DEBUG("\t\t[ -C filename ]  # Set tone curve\n");
 	DEBUG("\t\t[ -e ]           # Query error log\n");
@@ -646,6 +723,7 @@ static void kodak605_cmdline(void)
 	DEBUG("\t\t[ -R ]           # Reset printer to factory defaults\n");
 	DEBUG("\t\t[ -s ]           # Query status\n");
 	DEBUG("\t\t[ -X jobid ]     # Cancel job\n");
+//	DEBUG("\t\t[ -Z 0|1 ]       # Dump all parameters\n");
 }
 
 static int kodak605_cmdline_arg(void *vctx, int argc, char **argv)
@@ -656,14 +734,22 @@ static int kodak605_cmdline_arg(void *vctx, int argc, char **argv)
 	if (!ctx)
 		return -1;
 
-	while ((i = getopt(argc, argv, GETOPT_LIST_GLOBAL "c:C:eFil:L:mrRsX:")) >= 0) {
+	while ((i = getopt(argc, argv, GETOPT_LIST_GLOBAL "b:c:C:eFil:L:mrRsX:Z:")) >= 0) {
 		switch(i) {
 		GETOPT_PROCESS_GLOBAL
+		case 'b':
+			if (optarg[0] == '1')
+				j = sinfonia_button_set(&ctx->dev, BUTTON_ENABLED);
+			else if (optarg[0] == '0')
+				j = sinfonia_button_set(&ctx->dev, BUTTON_DISABLED);
+			else
+				return -1;
+			break;
 		case 'c':
 			j = sinfonia_gettonecurve(&ctx->dev, TONECURVE_USER, optarg);
 			break;
 		case 'C':
-			j = sinfonia_settonecurve(&ctx->dev, TONECURVE_USER, optarg);
+			j = sinfonia_settonecurve(&ctx->dev, UPDATE_TARGET_TONE_USER, optarg);
 			break;
 		case 'e':
 			j = sinfonia_geterrorlog(&ctx->dev);
@@ -678,7 +764,7 @@ static int kodak605_cmdline_arg(void *vctx, int argc, char **argv)
 			j = sinfonia_gettonecurve(&ctx->dev, TONECURVE_CURRENT, optarg);
 			break;
 		case 'L':
-			j = sinfonia_settonecurve(&ctx->dev, TONECURVE_CURRENT, optarg);
+			j = sinfonia_settonecurve(&ctx->dev, UPDATE_TARGET_TONE_CURRENT, optarg);
 			break;
 		case 'm':
 			kodak605_dump_mediainfo(ctx->media);
@@ -696,10 +782,13 @@ static int kodak605_cmdline_arg(void *vctx, int argc, char **argv)
 			if (!j)
 				kodak605_dump_status(ctx, &sts);
 			break;
+		}
 		case 'X':
 			j = sinfonia_canceljob(&ctx->dev, atoi(optarg));
 			break;
-		}
+		case 'Z':
+			j = sinfonia_dumpallparams(&ctx->dev, atoi(optarg));
+			break;
 		default:
 			break;  /* Ignore completely */
 		}
@@ -707,7 +796,7 @@ static int kodak605_cmdline_arg(void *vctx, int argc, char **argv)
 		if (j) return j;
 	}
 
-	return 0;
+	return CUPS_BACKEND_OK;
 }
 
 static int kodak605_query_markers(void *vctx, struct marker **markers, int *count)
@@ -721,22 +810,93 @@ static int kodak605_query_markers(void *vctx, struct marker **markers, int *coun
 
 	ctx->marker.levelnow = sts.donor;
 
-	*markers = &ctx->marker;
-	*count = 1;
+	if (markers) *markers = &ctx->marker;
+	if (count) *count = 1;
+
+	return CUPS_BACKEND_OK;
+}
+
+static int kodak605_query_stats(void *vctx,  struct printerstats *stats)
+{
+	struct kodak605_ctx *ctx = vctx;
+	struct kodak605_status status;
+
+	if (kodak605_query_markers(ctx, NULL, NULL))
+		return CUPS_BACKEND_FAILED;
+
+	if (kodak605_get_status(ctx, &status))
+		return CUPS_BACKEND_FAILED;
+
+	switch (ctx->dev.conn->type) {
+	case P_KODAK_605:
+		stats->mfg = "Kodak";
+		stats->model = "605";
+		break;
+	case P_KODAK_7000:
+		stats->mfg = "Kodak";
+		stats->model = "7000";
+		break;
+	case P_KODAK_701X:
+		stats->mfg = "Kodak";
+		stats->model = "7010/7015";
+		break;
+	default:
+		stats->mfg = "Unknown";
+		stats->model = "Unknown";
+		break;
+	}
+
+	if (sinfonia_query_serno(ctx->dev.conn,
+				 ctx->serial, sizeof(ctx->serial)))
+		return CUPS_BACKEND_FAILED;
+
+	stats->serial = ctx->serial;
+
+	{
+		struct sinfonia_fwinfo_cmd  cmd;
+		struct sinfonia_fwinfo_resp resp;
+		int num = 0;
+		cmd.hdr.cmd = cpu_to_le16(SINFONIA_CMD_FWINFO);
+		cmd.hdr.len = cpu_to_le16(1);
+		cmd.target = FWINFO_TARGET_MAIN_APP;
+
+		if (sinfonia_docmd(&ctx->dev,
+				   (uint8_t*)&cmd, sizeof(cmd),
+				   (uint8_t*)&resp, sizeof(resp),
+				   &num))
+			return CUPS_BACKEND_FAILED;
+		snprintf(ctx->fwver, sizeof(ctx->fwver)-1,
+			 "%d.%d", resp.major, resp.minor);
+		stats->fwver = ctx->fwver;
+	}
+
+	stats->decks = 1;
+	stats->mediatype[0] = ctx->marker.name;
+	stats->levelmax[0] = ctx->marker.levelmax;
+	stats->levelnow[0] = ctx->marker.levelnow;
+	stats->name[0] = "Roll";
+	if (status.hdr.status == ERROR_PRINTER) {
+		if(status.hdr.error == ERROR_NONE)
+			status.hdr.error = status.hdr.status;
+		stats->status[0] = strdup(sinfonia_error_str(status.hdr.error));
+	} else {
+		stats->status[0] = strdup(sinfonia_status_str(status.hdr.status));
+	}
+	stats->cnt_life[0] = le32_to_cpu(status.ctr_life);
 
 	return CUPS_BACKEND_OK;
 }
 
 static const char *kodak605_prefixes[] = {
 	"kodak605",  // Family driver, do NOT nuke.
-	"kodak-605", "kodak-7000", "kodak-7010", "kodak-7015",
+	"kodak-701x", // Just in case
 	NULL,
 };
 
 /* Exported */
-struct dyesub_backend kodak605_backend = {
+const struct dyesub_backend kodak605_backend = {
 	.name = "Kodak 605/70xx",
-	.version = "0.45" " (lib " LIBSINFONIA_VER ")",
+	.version = "0.55" " (lib " LIBSINFONIA_VER ")",
 	.uri_prefixes = kodak605_prefixes,
 	.cmdline_usage = kodak605_cmdline,
 	.cmdline_arg = kodak605_cmdline_arg,
@@ -746,11 +906,13 @@ struct dyesub_backend kodak605_backend = {
 	.read_parse = kodak605_read_parse,
 	.main_loop = kodak605_main_loop,
 	.query_markers = kodak605_query_markers,
+	.query_serno = sinfonia_query_serno,
+	.query_stats = kodak605_query_stats,
 	.devices = {
 		{ USB_VID_KODAK, USB_PID_KODAK_605, P_KODAK_605, "Kodak", "kodak-605"},
 		{ USB_VID_KODAK, USB_PID_KODAK_7000, P_KODAK_7000, "Kodak", "kodak-7000"},
 		{ USB_VID_KODAK, USB_PID_KODAK_7010, P_KODAK_701X, "Kodak", "kodak-7010"},
-		{ USB_VID_KODAK, USB_PID_KODAK_7015, P_KODAK_701X, "Kodak", "kodak-7015"},
+		{ USB_VID_KODAK, USB_PID_KODAK_7015, P_KODAK_701X, "Kodak", "kodak-7015"}, /* Duplicate */
 		{ 0, 0, 0, NULL, NULL}
 	}
 };
