@@ -1,7 +1,7 @@
 /*
  *   Kodak 6800/6850 Photo Printer CUPS backend -- libusb-1.0 version
  *
- *   (c) 2013-2019 Solomon Peachy <pizza@shaftnet.org>
+ *   (c) 2013-2020 Solomon Peachy <pizza@shaftnet.org>
  *
  *   Development of this backend was sponsored by:
  *
@@ -30,16 +30,6 @@
  *
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <unistd.h>
-
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <signal.h>
-
 #define BACKEND kodak6800_backend
 
 #include "backend_common.h"
@@ -103,11 +93,8 @@ struct kodak68x0_media_readback {
 
 /* Private data structure */
 struct kodak6800_ctx {
-	struct libusb_device_handle *dev;
-	uint8_t endp_up;
-	uint8_t endp_down;
+	struct dyesub_connection *conn;
 
-	int type;
 	int supports_sub4x6;
 
 	uint8_t jobid;
@@ -118,6 +105,9 @@ struct kodak6800_ctx {
 
 	struct kodak68x0_status_readback sts;
 
+	char serial[32];
+	char fwver[32];
+
 	struct marker marker;
 };
 
@@ -130,17 +120,17 @@ static int kodak6800_do_cmd(struct kodak6800_ctx *ctx,
         int ret;
 
         /* Write command */
-        if ((ret = send_data(ctx->dev, ctx->endp_down,
+        if ((ret = send_data(ctx->conn,
                              cmd, cmd_len)))
                 return (ret < 0) ? ret : -99;
 
         /* Read response */
-        ret = read_data(ctx->dev, ctx->endp_up,
+        ret = read_data(ctx->conn,
                         resp, resp_len, actual_len);
         if (ret < 0)
                 return ret;
 
-        return 0;
+        return CUPS_BACKEND_OK;
 }
 
 static void kodak68x0_dump_mediainfo(struct sinfonia_mediainfo_item *sizes,
@@ -164,6 +154,16 @@ static void kodak68x0_dump_mediainfo(struct sinfonia_mediainfo_item *sizes,
 	INFO("\n");
 }
 
+static void kodak6800_fillhdr(uint8_t *req, uint8_t cmd)
+{
+	req[0] = 0x03;  /* CMD type */
+	req[1] = 0x1b;  /* ESC */
+	req[2] = 0x43;  /* C */
+	req[3] = 0x48;  /* H */
+	req[4] = 0x43;  /* C */
+	req[5] = cmd;  /* Commmand code */
+}
+
 #define MAX_MEDIA_LEN (sizeof(struct kodak68x0_media_readback) + MAX_MEDIAS * sizeof(struct sinfonia_mediainfo_item))
 
 static int kodak6800_get_mediainfo(struct kodak6800_ctx *ctx)
@@ -181,13 +181,7 @@ static int kodak6800_get_mediainfo(struct kodak6800_ctx *ctx)
 
 	for (j = 0 ; j < 2 ; j ++) {
 		memset(media, 0, sizeof(*media));
-
-		req[0] = 0x03;
-		req[1] = 0x1b;
-		req[2] = 0x43;
-		req[3] = 0x48;
-		req[4] = 0x43;
-		req[5] = 0x1a;
+		kodak6800_fillhdr(req, 0x1a);
 		req[6] = j;
 
 		/* Issue command and get response */
@@ -228,13 +222,7 @@ static int kodak68x0_canceljob(struct kodak6800_ctx *ctx,
 	int ret, num;
 
 	memset(req, 0, sizeof(req));
-
-	req[0] = 0x03;
-	req[1] = 0x1b;
-	req[2] = 0x43;
-	req[3] = 0x48;
-	req[4] = 0x43;
-	req[5] = 0x13;
+	kodak6800_fillhdr(req, 0x13);
 	req[6] = id;
 
 	/* Issue command and get response */
@@ -249,7 +237,7 @@ static int kodak68x0_canceljob(struct kodak6800_ctx *ctx,
 		return -99;
 	}
 
-	return 0;
+	return CUPS_BACKEND_OK;
 }
 
 static int kodak68x0_reset(struct kodak6800_ctx *ctx)
@@ -258,12 +246,7 @@ static int kodak68x0_reset(struct kodak6800_ctx *ctx)
 	int ret, num;
 
 	memset(req, 0, sizeof(req));
-
-	req[0] = 0x03;
-	req[1] = 0x1b;
-	req[2] = 0x43;
-	req[3] = 0x48;
-	req[4] = 0xc0;
+	kodak6800_fillhdr(req, 0xc0);
 
 	/* Issue command and get response */
 	if ((ret = kodak6800_do_cmd(ctx, req, sizeof(req),
@@ -277,12 +260,12 @@ static int kodak68x0_reset(struct kodak6800_ctx *ctx)
 		return -99;
 	}
 
-	return 0;
+	return CUPS_BACKEND_OK;
 }
 
 static void kodak68x0_dump_status(struct kodak6800_ctx *ctx, struct kodak68x0_status_readback *status)
 {
-	char *detail;
+	const char *detail;
 
 	switch (status->status) {
         case STATUS_PRINTING:
@@ -329,21 +312,11 @@ static void kodak68x0_dump_status(struct kodak6800_ctx *ctx, struct kodak68x0_st
 	INFO("\tThermal Head  : %u\n", be32_to_cpu(status->maint));
 	INFO("\tCutter        : %u\n", be32_to_cpu(status->cutter));
 
-	if (ctx->type == P_KODAK_6850) {
-		int max;
+	if (ctx->conn->type == P_KODAK_6850) {
+		int max = kodak6_mediamax(ctx->media_type);
 
 		INFO("\tMedia         : %u\n", be32_to_cpu(status->media));
 
-		switch(ctx->media_type) {
-		case KODAK6_MEDIA_6R:
-		case KODAK6_MEDIA_6TR2:
-			max = 375;
-			break;
-		default:
-			max = 0;
-			break;
-		}
-
 		if (max) {
 			INFO("\t  Remaining   : %u\n", max - be32_to_cpu(status->media));
 		} else {
@@ -365,12 +338,7 @@ static int kodak6800_get_status(struct kodak6800_ctx *ctx,
 	memset(req, 0, sizeof(req));
 	memset(status, 0, sizeof(*status));
 
-	req[0] = 0x03;
-	req[1] = 0x1b;
-	req[2] = 0x43;
-	req[3] = 0x48;
-	req[4] = 0x43;
-	req[5] = 0x03;
+	kodak6800_fillhdr(req, 0x03);
 
 	/* Issue command and get response */
 	if ((ret = kodak6800_do_cmd(ctx, req, sizeof(req),
@@ -387,17 +355,17 @@ static int kodak6800_get_status(struct kodak6800_ctx *ctx,
 	/* Byteswap important stuff */
 	status->status2 = be32_to_cpu(status->status2);
 
-	return 0;
+	return CUPS_BACKEND_OK;
 }
 
-static int kodak6800_get_tonecurve(struct kodak6800_ctx *ctx, char *fname)
+static int kodak6800_get_tonecurve(struct kodak6800_ctx *ctx, uint8_t curve, char *fname)
 {
 	uint8_t cmdbuf[16];
 	uint8_t respbuf[64];
 	int ret, num = 0;
 	int i;
 
-	uint16_t *data = malloc(TONE_CURVE_SIZE);
+	uint16_t *data = malloc(TONE_CURVE_SIZE * sizeof(uint16_t));
 	if (!data) {
 		ERROR("Memory Allocation Failure\n");
 		return -1;
@@ -406,19 +374,14 @@ static int kodak6800_get_tonecurve(struct kodak6800_ctx *ctx, char *fname)
 	INFO("Dump Tone Curve to '%s'\n", fname);
 
 	/* Initial Request */
-	cmdbuf[0] = 0x03;
-	cmdbuf[1] = 0x1b;
-	cmdbuf[2] = 0x43;
-	cmdbuf[3] = 0x48;
-	cmdbuf[4] = 0x43;
-	cmdbuf[5] = 0x0c;
+	kodak6800_fillhdr(cmdbuf, 0x0c);
 	cmdbuf[6] = 0x54;
 	cmdbuf[7] = 0x4f;
 	cmdbuf[8] = 0x4e;
 	cmdbuf[9] = 0x45;
 	cmdbuf[10] = 0x72;
-	cmdbuf[11] = 0x01; /* 01 for user tonecurve, can be 00 or 02 */
-	cmdbuf[12] = 0x00; /* param table? */
+	cmdbuf[11] = curve;
+	cmdbuf[12] = PARAM_TABLE_NONE;
 	cmdbuf[13] = 0x00;
 	cmdbuf[14] = 0x00;
 	cmdbuf[15] = 0x00;
@@ -437,12 +400,7 @@ static int kodak6800_get_tonecurve(struct kodak6800_ctx *ctx, char *fname)
 	}
 
 	/* Then we can poll the data */
-	cmdbuf[0] = 0x03;
-	cmdbuf[1] = 0x1b;
-	cmdbuf[2] = 0x43;
-	cmdbuf[3] = 0x48;
-	cmdbuf[4] = 0x43;
-	cmdbuf[5] = 0x0c;
+	kodak6800_fillhdr(cmdbuf, 0x0c);
 	cmdbuf[6] = 0x54;
 	cmdbuf[7] = 0x4f;
 	cmdbuf[8] = 0x4e;
@@ -473,7 +431,7 @@ static int kodak6800_get_tonecurve(struct kodak6800_ctx *ctx, char *fname)
 			goto done;
 		}
 
-		for (i = 0 ; i < 768; i++) {
+		for (i = 0 ; i < TONE_CURVE_SIZE; i++) {
 			/* Byteswap appropriately */
 			data[i] = cpu_to_be16(le16_to_cpu(data[i]));
 			ret = write(tc_fd, &data[i], sizeof(uint16_t));
@@ -488,14 +446,14 @@ static int kodak6800_get_tonecurve(struct kodak6800_ctx *ctx, char *fname)
 	return ret;
 }
 
-static int kodak6800_set_tonecurve(struct kodak6800_ctx *ctx, char *fname)
+static int kodak6800_set_tonecurve(struct kodak6800_ctx *ctx, uint8_t curve, char *fname)
 {
 	uint8_t cmdbuf[64];
 	uint8_t respbuf[64];
 	int ret, num = 0;
 	int remain;
 
-	uint16_t *data = malloc(TONE_CURVE_SIZE);
+	uint16_t *data = malloc(TONE_CURVE_SIZE * sizeof(uint16_t));
 	uint8_t *ptr;
 
 	if (!data) {
@@ -506,30 +464,25 @@ static int kodak6800_set_tonecurve(struct kodak6800_ctx *ctx, char *fname)
 	INFO("Set Tone Curve from '%s'\n", fname);
 
 	/* Read in file */
-	if ((ret = dyesub_read_file(fname, data, TONE_CURVE_SIZE, NULL))) {
+	if ((ret = dyesub_read_file(fname, data, TONE_CURVE_SIZE * sizeof(uint16_t), NULL))) {
 		ERROR("Failed to read Tone Curve file\n");
 		goto done;
 	}
 
 	/* Byteswap data to printer's format */
-	for (ret = 0; ret < (TONE_CURVE_SIZE)/2 ; ret++) {
+	for (ret = 0; ret < TONE_CURVE_SIZE ; ret++) {
 		data[ret] = cpu_to_le16(be16_to_cpu(data[ret]));
 	}
 
 	/* Initial Request */
-	cmdbuf[0] = 0x03;
-	cmdbuf[1] = 0x1b;
-	cmdbuf[2] = 0x43;
-	cmdbuf[3] = 0x48;
-	cmdbuf[4] = 0x43;
-	cmdbuf[5] = 0x0c;
+	kodak6800_fillhdr(cmdbuf, 0x0c);
 	cmdbuf[6] = 0x54;
 	cmdbuf[7] = 0x4f;
 	cmdbuf[8] = 0x4e;
 	cmdbuf[9] = 0x45;
 	cmdbuf[10] = 0x77;
-	cmdbuf[11] = 0x01; /* User TC.  Can be 00 or 02 */
-	cmdbuf[12] = 0x00; /* param table? */
+	cmdbuf[11] = curve;
+	cmdbuf[12] = PARAM_TABLE_NONE;
 	cmdbuf[13] = 0x00;
 	cmdbuf[14] = 0x00;
 	cmdbuf[15] = 0x00;
@@ -553,7 +506,7 @@ static int kodak6800_set_tonecurve(struct kodak6800_ctx *ctx, char *fname)
 	}
 
 	ptr = (uint8_t*) data;
-	remain = TONE_CURVE_SIZE;
+	remain = TONE_CURVE_SIZE * sizeof(uint16_t);
 	while (remain > 0) {
 		int count = remain > 63 ? 63 : remain;
 
@@ -586,12 +539,10 @@ done:
 	return ret;
 }
 
-static int kodak6800_query_serno(struct libusb_device_handle *dev, uint8_t endp_up, uint8_t endp_down, char *buf, int buf_len)
+static int kodak6800_query_serno(struct dyesub_connection *conn, char *buf, int buf_len)
 {
 	struct kodak6800_ctx ctx = {
-		.dev = dev,
-		.endp_up = endp_up,
-		.endp_down = endp_down,
+		.conn = conn,
 	};
 
 	int ret;
@@ -603,12 +554,7 @@ static int kodak6800_query_serno(struct libusb_device_handle *dev, uint8_t endp_
 	memset(req, 0, sizeof(req));
 	memset(resp, 0, sizeof(resp));
 
-	req[0] = 0x03;
-	req[1] = 0x1b;
-	req[2] = 0x43;
-	req[3] = 0x48;
-	req[4] = 0x43;
-	req[5] = 0x12;
+	kodak6800_fillhdr(req, 0x12);
 
 	/* Issue command and get response */
 	if ((ret = kodak6800_do_cmd(&ctx, req, sizeof(req),
@@ -624,7 +570,7 @@ static int kodak6800_query_serno(struct libusb_device_handle *dev, uint8_t endp_
 	strncpy(buf, (char*)resp+24, buf_len);
 	buf[buf_len-1] = 0;
 
-	return 0;
+	return CUPS_BACKEND_OK;
 }
 
 static int kodak6850_send_unk(struct kodak6800_ctx *ctx)
@@ -634,12 +580,7 @@ static int kodak6850_send_unk(struct kodak6800_ctx *ctx)
 	int ret = 0, num = 0;
 
 	memset(cmdbuf, 0, sizeof(cmdbuf));
-	cmdbuf[0] = 0x03;
-	cmdbuf[1] = 0x1b;
-	cmdbuf[2] = 0x43;
-	cmdbuf[3] = 0x48;
-	cmdbuf[4] = 0x43;
-	cmdbuf[5] = 0x4c;
+	kodak6800_fillhdr(cmdbuf, 0x4c);
 
 	/* Issue command and get response */
 	if ((ret = kodak6800_do_cmd(ctx, cmdbuf, sizeof(cmdbuf),
@@ -658,20 +599,15 @@ static int kodak6850_send_unk(struct kodak6800_ctx *ctx)
 		return CUPS_BACKEND_FAILED;
 	}
 
-#if 0
-	// XXX No particular idea what this actually is
-	if (rdbuf[1] != 0x01 && rdbuf[1] != 0x00) {
-		ERROR("Unexpected status code (0x%02x)!\n", rdbuf[1]);
-		return CUPS_BACKEND_FAILED;
-	}
-#endif
 	return ret;
 }
 
 static void kodak6800_cmdline(void)
 {
-	DEBUG("\t\t[ -c filename ]  # Get tone curve\n");
-	DEBUG("\t\t[ -C filename ]  # Set tone curve\n");
+	DEBUG("\t\t[ -c filename ]  # Get user/NV tone curve\n");
+	DEBUG("\t\t[ -C filename ]  # Set user/NV tone curve\n");
+	DEBUG("\t\t[ -l filename ]  # Get current tone curve\n");
+	DEBUG("\t\t[ -L filename ]  # Set current tone curve\n");
 	DEBUG("\t\t[ -m ]           # Query media\n");
 	DEBUG("\t\t[ -s ]           # Query status\n");
 	DEBUG("\t\t[ -R ]           # Reset printer\n");
@@ -686,14 +622,20 @@ static int kodak6800_cmdline_arg(void *vctx, int argc, char **argv)
 	if (!ctx)
 		return -1;
 
-	while ((i = getopt(argc, argv, GETOPT_LIST_GLOBAL "C:c:mRsX:")) >= 0) {
+	while ((i = getopt(argc, argv, GETOPT_LIST_GLOBAL "C:c:L:l:mRsX:")) >= 0) {
 		switch(i) {
 		GETOPT_PROCESS_GLOBAL
 		case 'c':
-			j = kodak6800_get_tonecurve(ctx, optarg);
+			j = kodak6800_get_tonecurve(ctx, TONE_TABLE_USER, optarg);
 			break;
 		case 'C':
-			j = kodak6800_set_tonecurve(ctx, optarg);
+			j = kodak6800_set_tonecurve(ctx, TONE_TABLE_USER, optarg);
+			break;
+		case 'l':
+			j = kodak6800_get_tonecurve(ctx, TONE_TABLE_CURRENT, optarg);
+			break;
+		case 'L':
+			j = kodak6800_set_tonecurve(ctx, TONE_TABLE_CURRENT, optarg);
 			break;
 		case 'm':
 			kodak68x0_dump_mediainfo(ctx->sizes, ctx->media_count, ctx->media_type);
@@ -717,7 +659,7 @@ static int kodak6800_cmdline_arg(void *vctx, int argc, char **argv)
 		if (j) return j;
 	}
 
-	return 0;
+	return CUPS_BACKEND_OK;
 }
 
 static void *kodak6800_init(void)
@@ -732,15 +674,11 @@ static void *kodak6800_init(void)
 	return ctx;
 }
 
-static int kodak6800_attach(void *vctx, struct libusb_device_handle *dev, int type,
-			    uint8_t endp_up, uint8_t endp_down, uint8_t jobid)
+static int kodak6800_attach(void *vctx, struct dyesub_connection *conn, uint8_t jobid)
 {
 	struct kodak6800_ctx *ctx = vctx;
 
-	ctx->dev = dev;
-	ctx->endp_up = endp_up;
-	ctx->endp_down = endp_down;
-	ctx->type = type;
+	ctx->conn = conn;
 
         /* Ensure jobid is sane */
         ctx->jobid = jobid & 0x7f;
@@ -754,7 +692,7 @@ static int kodak6800_attach(void *vctx, struct libusb_device_handle *dev, int ty
 			return CUPS_BACKEND_FAILED;
 		}
 		uint16_t fw = be16_to_cpu(ctx->sts.main_fw);
-		if (ctx->type == P_KODAK_6850) {
+		if (ctx->conn->type == P_KODAK_6850) {
 			if ((fw >= 878) ||
 			    (fw < 800 && fw >= 678)) {
 				ctx->supports_sub4x6 = 1;
@@ -787,8 +725,9 @@ static int kodak6800_attach(void *vctx, struct libusb_device_handle *dev, int ty
 
 	ctx->marker.color = "#00FFFF#FF00FF#FFFF00";
 	ctx->marker.name = kodak6_mediatypes(ctx->media_type);
+	ctx->marker.numtype = ctx->media_type;
 	ctx->marker.levelmax = 100; /* Ie percentage */
-	ctx->marker.levelnow = -2;
+	ctx->marker.levelnow = CUPS_MARKER_UNKNOWN;
 
 	return CUPS_BACKEND_OK;
 }
@@ -904,7 +843,7 @@ static int kodak6800_read_parse(void *vctx, const void **vjob, int data_fd, int
 
 	hdr.copies = be16_to_cpu(hdr.copies);
 	hdr.copies = packed_bcd_to_uint32((char*)&hdr.copies, 2);
-	if (hdr.copies > 1)
+	if (hdr.copies > copies)
 		copies = hdr.copies;
 
 	/* Fill out job structure */
@@ -984,7 +923,7 @@ static int kodak6800_main_loop(void *vctx, const void *vjob) {
 	}
 
 	/* This command is unknown, sort of a secondary status query */
-	if (ctx->type == P_KODAK_6850) {
+	if (ctx->conn->type == P_KODAK_6850) {
 		ret = kodak6850_send_unk(ctx);
 		if (ret)
 			return ret;
@@ -996,12 +935,7 @@ static int kodak6800_main_loop(void *vctx, const void *vjob) {
 
 	/* Fill out printjob header */
 	struct kodak6800_hdr hdr;
-	hdr.hdr[0] = 0x03;
-	hdr.hdr[1] = 0x1b;
-	hdr.hdr[2] = 0x43;
-	hdr.hdr[3] = 0x48;
-	hdr.hdr[4] = 0x43;
-	hdr.hdr[5] = 0x0a;
+	kodak6800_fillhdr(hdr.hdr, 0x0a);
 	hdr.hdr[6] = 0x00;
 	hdr.jobid = ctx->jobid;
 	hdr.copies = uint16_to_packed_bcd(copies);
@@ -1024,7 +958,7 @@ static int kodak6800_main_loop(void *vctx, const void *vjob) {
 
 //	sleep(1); // Appears to be necessary for reliability
 	INFO("Sending image data\n");
-	if ((send_data(ctx->dev, ctx->endp_down,
+	if ((send_data(ctx->conn,
 			     job->databuf, job->datalen)) != 0)
 		return CUPS_BACKEND_FAILED;
 
@@ -1074,24 +1008,67 @@ static int kodak6800_query_markers(void *vctx, struct marker **markers, int *cou
 
 	ctx->marker.levelnow = ctx->sts.donor;
 
-	*markers = &ctx->marker;
-	*count = 1;
+	if (markers) *markers = &ctx->marker;
+	if (count) *count = 1;
+
+	return CUPS_BACKEND_OK;
+}
+
+static int kodak6800_query_stats(void *vctx,  struct printerstats *stats)
+{
+	struct kodak6800_ctx *ctx = vctx;
+
+	if (kodak6800_query_markers(ctx, NULL, NULL))
+		return CUPS_BACKEND_FAILED;
+
+	switch (ctx->conn->type) {
+	case P_KODAK_6800:
+		stats->mfg = "Kodak";
+		stats->model = "6800";
+		break;
+	case P_KODAK_6850:
+		stats->mfg = "Kodak";
+		stats->model = "6850";
+		break;
+	default:
+		stats->mfg = "Unknown";
+		stats->model = "Unknown";
+		break;
+	}
+
+	if (kodak6800_query_serno(ctx->conn,
+				  ctx->serial, sizeof(ctx->serial)))
+		return CUPS_BACKEND_FAILED;
+
+	stats->serial = ctx->serial;
+
+	snprintf(ctx->fwver, sizeof(ctx->fwver)-1,
+		 "%d / %d", be16_to_cpu(ctx->sts.main_fw),
+		 be16_to_cpu(ctx->sts.dsp_fw));
+	stats->fwver = ctx->fwver;
+
+	stats->decks = 1;
+	stats->mediatype[0] = ctx->marker.name;
+	stats->levelmax[0] = ctx->marker.levelmax;
+	stats->levelnow[0] = ctx->marker.levelnow;
+	stats->name[0] = "Roll";
+	stats->status[0] = strdup(sinfonia_1x45_status_str(ctx->sts.status1, ctx->sts.status2, ctx->sts.errcode));
+	stats->cnt_life[0] = be32_to_cpu(ctx->sts.lifetime);
 
 	return CUPS_BACKEND_OK;
 }
 
 static const char *kodak6800_prefixes[] = {
 	"kodak68x0", // Family driver, do not nuke.
-	"kodak-6800", "kodak-6850",
 	// Backwards-compatibility
 	"kodak6800", "kodak6850",
 	NULL
 };
 
 /* Exported */
-struct dyesub_backend kodak6800_backend = {
+const struct dyesub_backend kodak6800_backend = {
 	.name = "Kodak 6800/6850",
-	.version = "0.73" " (lib " LIBSINFONIA_VER ")",
+	.version = "0.80" " (lib " LIBSINFONIA_VER ")",
 	.uri_prefixes = kodak6800_prefixes,
 	.cmdline_usage = kodak6800_cmdline,
 	.cmdline_arg = kodak6800_cmdline_arg,
@@ -1102,6 +1079,7 @@ struct dyesub_backend kodak6800_backend = {
 	.main_loop = kodak6800_main_loop,
 	.query_serno = kodak6800_query_serno,
 	.query_markers = kodak6800_query_markers,
+	.query_stats = kodak6800_query_stats,
 	.devices = {
 		{ USB_VID_KODAK, USB_PID_KODAK_6800, P_KODAK_6800, "Kodak", "kodak-6800"},
 		{ USB_VID_KODAK, USB_PID_KODAK_6850, P_KODAK_6850, "Kodak", "kodak-6850"},
