@@ -1,7 +1,7 @@
 /*
  *   Magicard card printer family CUPS backend -- libusb-1.0 version
  *
- *   (c) 2017-2018 Solomon Peachy <pizza@shaftnet.org>
+ *   (c) 2017-2020 Solomon Peachy <pizza@shaftnet.org>
  *
  *   The latest version of this program can be found at:
  *
@@ -26,22 +26,12 @@
  *
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <unistd.h>
-#include <time.h>
-
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <signal.h>
-#include <string.h>
-
 #define BACKEND magicard_backend
 
 #include "backend_common.h"
 
+#include <time.h>
+
 /* Exported */
 #define USB_VID_MAGICARD     0x0C1F
 #define USB_PID_MAGICARD_TANGO2E 0x1800
@@ -118,11 +108,7 @@ struct magicard_printjob {
 
 /* Private data structure */
 struct magicard_ctx {
-	struct libusb_device_handle *dev;
-	uint8_t endp_up;
-	uint8_t endp_down;
-	int type;
-
+	struct dyesub_connection *conn;
 	struct marker marker;
 };
 
@@ -150,8 +136,8 @@ struct magicard_resp_header {
 };
 
 struct magicard_requests {
-	char *key;
-	char *desc;
+	const char *key;
+	const char *desc;
 	uint8_t type;
 };
 
@@ -210,7 +196,7 @@ static struct magicard_requests magicard_sta_requests[] = {
 
 /* Helper functions */
 static int magicard_build_cmd(uint8_t *buf,
-			       char *cmd, char *subcmd, char *arg)
+			      const char *cmd, const char *subcmd, const char *arg)
 {
 	struct magicard_cmd_header *hdr = (struct magicard_cmd_header *) buf;
 
@@ -229,7 +215,7 @@ static int magicard_build_cmd(uint8_t *buf,
 }
 
 static int magicard_build_cmd_simple(uint8_t *buf,
-				     char *cmd)
+				     const char *cmd)
 {
 	struct magicard_cmd_simple_header *hdr = (struct magicard_cmd_simple_header *) buf;
 	int len = strlen(cmd);
@@ -266,13 +252,13 @@ static int magicard_query_sensors(struct magicard_ctx *ctx)
 		snprintf(buf2, sizeof(buf2), "SNR%d", i);
 		ret = magicard_build_cmd_simple(buf, buf2);
 
-		if ((ret = send_data(ctx->dev, ctx->endp_down,
+		if ((ret = send_data(ctx->conn,
 				     buf, ret)))
 			return ret;
 
 		memset(buf, 0, sizeof(buf));
 
-		ret = read_data(ctx->dev, ctx->endp_up,
+		ret = read_data(ctx->conn,
 				buf, sizeof(buf), &num);
 
 		if (ret < 0)
@@ -284,7 +270,7 @@ static int magicard_query_sensors(struct magicard_ctx *ctx)
 		buf[num] = 0;
 		INFO("%s\n", buf);
 	}
-	return 0;
+	return CUPS_BACKEND_OK;
 }
 
 static int magicard_selftest_card(struct magicard_ctx *ctx)
@@ -294,7 +280,7 @@ static int magicard_selftest_card(struct magicard_ctx *ctx)
 
 	ret = magicard_build_cmd_simple(buf, "TST,");
 
-	ret = send_data(ctx->dev, ctx->endp_down,
+	ret = send_data(ctx->conn,
 			buf, ret);
 	return ret;
 }
@@ -306,7 +292,7 @@ static int magicard_reset(struct magicard_ctx *ctx)
 
 	ret = magicard_build_cmd_simple(buf, "RST,");
 
-	ret = send_data(ctx->dev, ctx->endp_down,
+	ret = send_data(ctx->conn,
 			buf, ret);
 	return ret;
 }
@@ -318,7 +304,7 @@ static int magicard_eject(struct magicard_ctx *ctx)
 
 	ret = magicard_build_cmd_simple(buf, "EJT,");
 
-	ret = send_data(ctx->dev, ctx->endp_down,
+	ret = send_data(ctx->conn,
 			buf, ret);
 	return ret;
 }
@@ -336,13 +322,13 @@ static int magicard_query_printer(struct magicard_ctx *ctx)
 		snprintf(buf2, sizeof(buf2), "QPR%d", i);
 		ret = magicard_build_cmd_simple(buf, buf2);
 
-		if ((ret = send_data(ctx->dev, ctx->endp_down,
+		if ((ret = send_data(ctx->conn,
 				     buf, ret)))
 			return ret;
 
 		memset(buf, 0, sizeof(buf));
 
-		ret = read_data(ctx->dev, ctx->endp_up,
+		ret = read_data(ctx->conn,
 				buf, sizeof(buf), &num);
 
 		if (ret < 0)
@@ -354,7 +340,7 @@ static int magicard_query_printer(struct magicard_ctx *ctx)
 		buf[num] = 0;
 		INFO("%s\n", buf);
 	}
-	return 0;
+	return CUPS_BACKEND_OK;
 }
 
 static int magicard_query_status(struct magicard_ctx *ctx)
@@ -374,13 +360,13 @@ static int magicard_query_status(struct magicard_ctx *ctx)
 		ret = magicard_build_cmd(buf, "REQ", "STA",
 				   magicard_sta_requests[i].key);
 
-		if ((ret = send_data(ctx->dev, ctx->endp_down,
+		if ((ret = send_data(ctx->conn,
 				     buf, ret)))
 			return ret;
 
 		memset(buf, 0, sizeof(buf));
 
-		ret = read_data(ctx->dev, ctx->endp_up,
+		ret = read_data(ctx->conn,
 				buf, sizeof(buf), &num);
 
 		if (ret < 0)
@@ -438,29 +424,26 @@ static void* magicard_init(void)
 {
 	struct magicard_ctx *ctx = malloc(sizeof(struct magicard_ctx));
 	if (!ctx) {
-		ERROR("Memory Allocation Failure!");
+		ERROR("Memory Allocation Failure!\n");
 		return NULL;
 	}
 	memset(ctx, 0, sizeof(struct magicard_ctx));
 	return ctx;
 }
 
-static int magicard_attach(void *vctx, struct libusb_device_handle *dev, int type,
-			   uint8_t endp_up, uint8_t endp_down, uint8_t jobid)
+static int magicard_attach(void *vctx, struct dyesub_connection *conn, uint8_t jobid)
 {
 	struct magicard_ctx *ctx = vctx;
 
 	UNUSED(jobid);
 
-	ctx->dev = dev;
-	ctx->endp_up = endp_up;
-	ctx->endp_down = endp_down;
-	ctx->type = type;
+	ctx->conn = conn;
 
 	ctx->marker.color = "#00FFFF#FF00FF#FFFF00";  // XXX YMCK too!
 	ctx->marker.name = "Unknown"; // LC1/LC3/LC6/LC8
-	ctx->marker.levelmax = -1;
-	ctx->marker.levelnow = -2;
+	ctx->marker.numtype = -1;
+	ctx->marker.levelmax = CUPS_MARKER_UNAVAILABLE;
+	ctx->marker.levelnow = CUPS_MARKER_UNKNOWN;
 
 	return CUPS_BACKEND_OK;
 }
@@ -858,11 +841,11 @@ static int magicard_main_loop(void *vctx, const void *vjob) {
 
 	copies = job->copies;
 top:
-	if ((ret = send_data(ctx->dev, ctx->endp_down,
+	if ((ret = send_data(ctx->conn,
 			     job->databuf, job->hdr_len)))
 		return CUPS_BACKEND_FAILED;
 
-	if ((ret = send_data(ctx->dev, ctx->endp_down,
+	if ((ret = send_data(ctx->conn,
 			     job->databuf + job->hdr_len, job->datalen - job->hdr_len)))
 		return CUPS_BACKEND_FAILED;
 
@@ -923,7 +906,7 @@ static int magicard_cmdline_arg(void *vctx, int argc, char **argv)
 		if (j) return j;
 	}
 
-	return 0;
+	return CUPS_BACKEND_OK;
 }
 
 static int magicard_query_markers(void *vctx, struct marker **markers, int *count)
@@ -938,17 +921,14 @@ static int magicard_query_markers(void *vctx, struct marker **markers, int *coun
 
 static const char *magicard_prefixes[] = {
 	"magicard", // Family name
-	"magicard-tango-2e", "magicard-enduro", "magicard-enduroplus",
-	// extras
-	"magicard-rio-2e",
 	// backwards compatibility
 	"tango2e", "enduro", "enduroplus",
 	NULL
 };
 
-struct dyesub_backend magicard_backend = {
+const struct dyesub_backend magicard_backend = {
 	.name = "Magicard family",
-	.version = "0.16",
+	.version = "0.17",
 	.uri_prefixes = magicard_prefixes,
 	.cmdline_arg = magicard_cmdline_arg,
 	.cmdline_usage = magicard_cmdline,
@@ -960,6 +940,7 @@ struct dyesub_backend magicard_backend = {
 	.query_markers = magicard_query_markers,
 	.devices = {
 		{ USB_VID_MAGICARD, USB_PID_MAGICARD_TANGO2E, P_MAGICARD, NULL, "magicard-tango2e"},
+//		{ USB_VID_MAGICARD, USB_PID_MAGICARD_TANGO2E, P_MAGICARD, NULL, "magicard-rio2e"},
 		{ USB_VID_MAGICARD, USB_PID_MAGICARD_ENDURO, P_MAGICARD, NULL, "magicard-enduro"},
 		{ USB_VID_MAGICARD, USB_PID_MAGICARD_ENDUROPLUS, P_MAGICARD, NULL, "magicard-enduroplus"},
 		{ USB_VID_MAGICARD, 0xFFFF, P_MAGICARD, NULL, "magicard"},
