@@ -1,7 +1,7 @@
  /*
  *   Shinko/Sinfonia Common Code
  *
- *   (c) 2019 Solomon Peachy <pizza@shaftnet.org>
+ *   (c) 2019-2020 Solomon Peachy <pizza@shaftnet.org>
  *
  *   The latest version of this program can be found at:
  *
@@ -18,16 +18,13 @@
  *   for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- *          [http://www.gnu.org/licenses/gpl-2.0.html]
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>.
  *
  *   SPDX-License-Identifier: GPL-2.0+
  *
  */
 
-#define LIBSINFONIA_VER "0.08"
+#define LIBSINFONIA_VER "0.17"
 
 #define SINFONIA_HDR1_LEN 0x10
 #define SINFONIA_HDR2_LEN 0x64
@@ -50,13 +47,19 @@ struct sinfonia_job_param {
 
 	uint32_t ext_flags;
 };
+#define EXT_FLAG_PLANARYMC  0x01
+#define EXT_FLAG_BACKPRINT  0x02
+#define EXT_FLAG_DOUBLESLUG 0x04
 
 struct sinfonia_printjob {
+	size_t jobsize;
+	int copies;
+	int can_combine;
+
 	struct sinfonia_job_param jp;
 
 	uint8_t *databuf;
 	int datalen;
-	int copies;
 };
 
 int sinfonia_read_parse(int data_fd, uint32_t model,
@@ -67,12 +70,19 @@ int sinfonia_raw18_read_parse(int data_fd, struct sinfonia_printjob *job);
 int sinfonia_raw28_read_parse(int data_fd, struct sinfonia_printjob *job);
 void sinfonia_cleanup_job(const void *vjob);
 
+
+/* mapping param IDs to names */
+struct sinfonia_param {
+	const uint8_t id;
+	const char *descr;
+};
+
 /* Common usb functions */
 struct sinfonia_usbdev {
-	struct libusb_device_handle *dev;
-	uint8_t endp_up;
-	uint8_t endp_down;
-	int type;
+	struct dyesub_connection *conn;
+
+	const struct sinfonia_param *params;
+	int params_count;
 
 	char const *(*error_codes)(uint8_t major, uint8_t minor);
 };
@@ -89,6 +99,11 @@ int sinfonia_geterrorlog(struct sinfonia_usbdev *usbh);
 int sinfonia_resetcurve(struct sinfonia_usbdev *usbh, int target, int id);
 int sinfonia_gettonecurve(struct sinfonia_usbdev *usbh, int type, char *fname);
 int sinfonia_settonecurve(struct sinfonia_usbdev *usbh, int target, char *fname);
+int sinfonia_button_set(struct sinfonia_usbdev *dev, int enable);
+
+int sinfonia_query_serno(struct dyesub_connection *conn, char *buf, int buf_len);
+int sinfonia_dumpallparams(struct sinfonia_usbdev *usbh, int known);
+const char *sinfonia_paramname(struct sinfonia_usbdev *usbh, int id);
 
 #define BANK_STATUS_FREE  0x00
 #define BANK_STATUS_XFER  0x01
@@ -97,14 +112,17 @@ int sinfonia_settonecurve(struct sinfonia_usbdev *usbh, int target, char *fname)
 
 const char *sinfonia_bank_statuses(uint8_t v);
 
-#define UPDATE_TARGET_USER    0x03
-#define UPDATE_TARGET_CURRENT 0x04
+#define UPDATE_TARGET_TONE_USER     0x03
+#define UPDATE_TARGET_TONE_CURRENT  0x04
+#define UPDATE_TARGET_LAM_USER 0x10
+#define UPDATE_TARGET_LAM_DEF  0x11
+#define UPDATE_TARGET_LAM_CUR  0x12
 
 /* Update is three channels, Y, M, C;
    each is 256 entries of 11-bit data padded to 16-bits.
    Printer expects LE data.  We use BE data on disk.
 */
-#define TONE_CURVE_SIZE 0x600
+#define TONE_CURVE_SIZE (256*3)
 const char *sinfonia_update_targets (uint8_t v);
 
 #define TONECURVE_INIT    0x00
@@ -124,7 +142,7 @@ struct sinfonia_error_item {
 #define ERROR_MAIN_APP_INACTIVE 0x02
 #define ERROR_COMMS_TIMEOUT     0x03
 #define ERROR_MAINT_NEEDED      0x04
-#define ERROR_BAD_COMMAND       0x05
+#define ERROR_INAPP_COMMAND     0x05
 #define ERROR_PRINTER           0x11
 #define ERROR_BUFFER_FULL       0x21
 
@@ -146,6 +164,7 @@ const char *sinfonia_print_modes(uint8_t v);
 #define PRINT_METHOD_COMBO_2 0x02
 #define PRINT_METHOD_COMBO_3 0x03 // S6245 only
 #define PRINT_METHOD_SPLIT   0x04
+#define PRINT_METHOD_COMBO_4 0x05 // S2245 only
 #define PRINT_METHOD_DOUBLE  0x08 // S6145 only
 #define PRINT_METHOD_DISABLE_ERR 0x10 // S6245 only
 #define PRINT_METHOD_NOTRIM  0x80 // S6145 only
@@ -173,7 +192,7 @@ struct sinfonia_status_hdr {
 	uint8_t  printer_major;
 	uint8_t  printer_minor;
 	uint8_t  reserved[2];
-	uint8_t  mode;  /* S6245 and EK605 only, so far */
+	uint8_t  mode;  /* S2245, S6245 and EK605 only, so far */
 	uint8_t  status;
 	uint16_t payload_len;
 } __attribute__((packed));
@@ -204,6 +223,12 @@ struct sinfonia_errorlog_resp {
 	struct sinfonia_error_item items[10];  /* Not all necessarily used */
 } __attribute__((packed));
 
+
+struct sinfonia_errorlog2_cmd {
+	struct sinfonia_cmd_hdr hdr;
+	uint16_t index;  /* 0 is latest */
+} __attribute__((packed));
+
 struct sinfonia_mediainfo_item {
 	uint8_t  code;
 	uint16_t columns;
@@ -214,6 +239,19 @@ struct sinfonia_mediainfo_item {
 	uint8_t  reserved[2];
 } __attribute__((packed));
 
+struct sinfonia_6x45_mediainfo_resp {
+	struct sinfonia_status_hdr hdr;
+	uint8_t ribbon_code;
+	uint8_t reserved;
+	uint8_t count;
+	struct sinfonia_mediainfo_item items[10];  /* Not all necessarily used */
+} __attribute__((packed));
+
+/* resp needs to be at least sizeof(struct sinfonia_6x45_mediainfo_resp) */
+
+int sinfonia_query_media(struct sinfonia_usbdev *usbh,
+			 void *resp);
+
 struct sinfonia_setparam_cmd {
 	struct sinfonia_cmd_hdr hdr;
 	uint8_t target;
@@ -256,6 +294,46 @@ struct sinfonia_getprintidstatus_resp {
 #define IDSTATUS_COMPLETED 0x0200
 #define IDSTATUS_ERROR     0xFFFF
 
+struct sinfonia_status_resp {
+	struct sinfonia_status_hdr hdr;
+	uint32_t count_lifetime;
+	uint32_t count_maint;
+	uint32_t count_paper;
+	uint32_t count_cutter;
+	uint32_t count_head;
+	uint32_t count_ribbon_left;
+	uint32_t reserved;
+
+	uint8_t  bank1_printid;
+	uint16_t bank1_remaining;
+	uint16_t bank1_finished;
+	uint16_t bank1_specified;
+	uint8_t  bank1_status;
+
+	uint8_t  bank2_printid;
+	uint16_t bank2_remaining;
+	uint16_t bank2_finished;
+	uint16_t bank2_specified;
+	uint8_t  bank2_status;
+
+	uint8_t  reserved2[16];
+	uint8_t  tonecurve_status;
+	uint8_t  reserved3[6];
+} __attribute__((packed));
+
+struct sinfonia_geteeprom_resp {
+	struct sinfonia_status_hdr hdr;
+	uint8_t data[256];
+} __attribute__((packed));
+
+struct sinfonia_button_cmd {
+	struct sinfonia_cmd_hdr hdr;
+	uint8_t  enabled;
+} __attribute__((packed));
+
+#define BUTTON_ENABLED  0x01
+#define BUTTON_DISABLED 0x00
+
 struct sinfonia_reset_cmd {
 	struct sinfonia_cmd_hdr hdr;
 	uint8_t  target;
@@ -264,7 +342,6 @@ struct sinfonia_reset_cmd {
 
 #define RESET_PRINTER       0x03
 #define RESET_TONE_CURVE    0x04
-
 #define TONE_CURVE_ID       0x01
 
 struct sinfonia_readtone_cmd {
@@ -283,11 +360,11 @@ struct sinfonia_readtone_resp {
 
 struct sinfonia_update_cmd {
 	struct sinfonia_cmd_hdr hdr;
-	uint8_t  target;
-	uint8_t  curve_id;
-	uint8_t  reset; // ??
+	uint8_t  target;    // UPDATE_TARGET_TONE_*
+	uint8_t  curve_id;  // 00 for lamination, 01 for tone?
+	uint8_t  reset;     // ??
 	uint8_t  reserved[3];
-	uint32_t size;
+	uint32_t size;  // TONE_CURVE_SIZE or lamination data that is rows*cols bytes
 } __attribute__((packed));
 
 struct sinfonia_getserial_resp {
@@ -325,7 +402,9 @@ struct sinfonia_printcmd18_hdr {
 	uint16_t copies;
 	uint16_t columns;
 	uint16_t rows;
-	uint8_t  reserved[8]; // columns and rows repeated, then nulls
+	uint16_t columns2;
+	uint16_t rows2;
+	uint8_t  reserved[4]; // then nulls
 	uint8_t  oc_mode;
 	uint8_t  method;
 	uint8_t  media; // reserved?
@@ -337,11 +416,53 @@ struct sinfonia_printcmd28_hdr {
 	uint16_t copies;
 	uint16_t columns;
 	uint16_t rows;
-	uint8_t  media;
+	uint8_t  media;  /* always 0 on S2245 */
 	uint8_t  reserved[7];
 	uint8_t  options;
 	uint8_t  method;
-	uint8_t  reserved2[11];
+	uint8_t  ipp;
+	uint8_t  reserved2[10];
+} __attribute__((packed));
+
+#define SINFONIA_PRINT28_OPTIONS_HQ   0x08
+#define SINFONIA_PRINT28_OC_MASK      0x03
+#define SINFONIA_PRINT28_OC_GLOSS     0x01
+#define SINFONIA_PRINT28_OC_MATTE     0x02
+
+#define SINFONIA_PRINT28_METHOD_PRINT_MASK   0x07
+#define SINFONIA_PRINT28_METHOD_PRINT_COMBO  0x02
+#define SINFONIA_PRINT28_METHOD_PRINT_SPLIT  0x04
+#define SINFONIA_PRINT28_METHOD_PRINT_4SPLIT 0x05
+
+#define SINFONIA_PRINT28_METHOD_ERR_RECOVERY 0x08
+#define SINFONIA_PRINT28_METHOD_ERR_PREHEAT  0x10
+
+#define SINFONIA_PRINT28_IPP_RESP     0x01
+#define SINFONIA_PRINT28_IPP_CONTOUR  0x02
+
+struct sinfonia_settime_cmd {
+	struct sinfonia_cmd_hdr hdr;
+	uint8_t enable;  /* 0 or 1 */
+	uint8_t second;
+	uint8_t minute;
+	uint8_t hour;
+	uint8_t day;
+	uint8_t month;
+	uint8_t year;
+} __attribute__((packed));
+
+struct kodak701x_backprint {
+	struct sinfonia_cmd_hdr hdr;
+	uint8_t unk_0;  // unknown.  maybe the line number?
+	uint8_t null[6]; // always zero.
+	uint8_t unk_1;  // length of text?  (max 40)
+	uint8_t text[42]; //
+} __attribute__((packed));
+
+struct kodak8810_cutlist {
+	struct sinfonia_cmd_hdr hdr;
+	uint8_t  entries; /* max 24 */
+	uint16_t cut[36]; /* LE, row number to cut at. Deltas must be >= 12. */
 } __attribute__((packed));
 
 #define CODE_4x6     0x00
@@ -402,49 +523,68 @@ const char *sinfonia_status_str(uint8_t v);
 
 #define SINFONIA_CMD_GETSTATUS  0x0001
 #define SINFONIA_CMD_MEDIAINFO  0x0002
-#define SINFONIA_CMD_MODELNAME  0x0003 // 2145 only
+#define SINFONIA_CMD_MODELNAME  0x0003 // 2145
 #define SINFONIA_CMD_ERRORLOG   0x0004
 #define SINFONIA_CMD_GETPARAM   0x0005 // !2145
 #define SINFONIA_CMD_GETSERIAL  0x0006 // !2145
 #define SINFONIA_CMD_PRINTSTAT  0x0007 // !2145
 #define SINFONIA_CMD_EXTCOUNTER 0x0008 // !2145
 
-#define SINFONIA_CMD_MEMORYBANK 0x000A // Brava 21 only?
+#define SINFONIA_CMD_MEMORYBANK 0x000A // 6145
 
 #define SINFONIA_CMD_PRINTJOB   0x4001
 #define SINFONIA_CMD_CANCELJOB  0x4002
 #define SINFONIA_CMD_FLASHLED   0x4003
 #define SINFONIA_CMD_RESET      0x4004
 #define SINFONIA_CMD_READTONE   0x4005
-#define SINFONIA_CMD_BUTTON     0x4006 // 2145 only
-#define SINFONIA_CMD_SETPARAM   0x4007
+#define SINFONIA_CMD_BUTTON     0x4006 // 2145?
+#define SINFONIA_CMD_SETPARAM   0x4007 // !2145
 
-#define SINFONIA_CMD_GETUNIQUE  0x8003 // 2145 only
+#define SINFONIA_CMD_SETLAMSTR  0x4008 // EK70xx, EK8810? (len 28)
+#define SINFONIA_CMD_COMMPPA    0x4009 // EK70xx
+#define SINFONIA_CMD_SETCUTLIST 0x4009 // EK8810 (len 73, count + 36 entries, 16bit LE)
+#define SINFONIA_CMD_SETPPAPARM 0x400A // EK70xx
+#define SINFONIA_CMD_WAKEUPSDBY 0x400A // EK8810 (len 1)
+#define SINFONIA_CMD_BACKPRINT  0x400B // EK701x only! (len 50)
+#define SINFONIA_CMD_UNKNOWN4C  0x400C // EK8810, panorama setup?
 
-#define SINFONIA_CMD_GETCORR    0x400D
-#define SINFONIA_CMD_GETEEPROM  0x400E
-#define SINFONIA_CMD_SETEEPROM  0x400F
-#define SINFONIA_CMD_SETTIME    0x4011 // 6245 only
+#define SINFONIA_CMD_GETCORR    0x400D // 6145/2245
+#define SINFONIA_CMD_GETEEPROM  0x400E // 6x45
+#define SINFONIA_CMD_SETEEPROM  0x400F // 6x45
 
-#define SINFONIA_CMD_DIAGNOSTIC 0xC001 // ??
+#define SINFONIA_CMD_SETTIME    0x4011 // 6245, 2245
+
+#define SINFONIA_CMD_UNIVERSAL  0x4080 // EK70xx
+
+#define SINFONIA_CMD_USBFWDL    0x8001 // EK70xx (len 5)
+#define SINFONIA_CMD_MAINTPERM  0x8002 // EK70xx
+#define SINFONIA_CMD_GETUNIQUE  0x8003 // 2145
+
+#define SINFONIA_CMD_SELFDIAG   0xC001 // (len 3)
+#define SINFONIA_CMD_DIAGRES    0xC002
 #define SINFONIA_CMD_FWINFO     0xC003
 #define SINFONIA_CMD_UPDATE     0xC004
-#define SINFONIA_CMD_SETUNIQUE  0xC007 // 2145 only
+#define SINFONIA_CMD_GETEEPROM2 0xC005 // EK70xx
+#define SINFONIA_CMD_SETEEPROM2 0xC006 // EK70xx
+#define SINFONIA_CMD_SETUNIQUE  0xC007 // 2145
+#define SINFONIA_CMD_RESETERR   0xC008
+#define SINFONIA_CMD_GETSERIAL2 0xC009 // EK70xx (len 8)
 
 const char *sinfonia_cmd_names(uint16_t v);
 
-//#define KODAK6_MEDIA_5R      // 189-9160
+#define KODAK6_MEDIA_5R   0xff //XX 189-9160
 #define KODAK6_MEDIA_6R   0x0b // 197-4096  [ Also: 101-0867, 141-9597, 659-9054, 169-6418, DNP-900-060 ]
 #define KODAK6_MEDIA_UNK  0x03 // ??? reported but unknown
 #define KODAK6_MEDIA_6TR2 0x2c // 396-2941
 //#define KODAK6_MEDIA_5FR2    // 6900-compatible
 //#define KODAK6_MEDIA_6FR2    // 6900-compatible, 102-5925
 #define KODAK6_MEDIA_NONE 0x00
-//#define KODAK7_MEDIA_5R      // 164-9011 137-0600
+#define KODAK7_MEDIA_5R   0xfe //XX 164-9011 137-0600
 #define KODAK7_MEDIA_6R   0x29 // 659-9047 166-1925 396-2966 846-2004 103-7688 DNP-900-070 -- ALSO FUJI R68-D2P570 16578944
 //#define KODAK7_MEDIA_6TA2
 //#define KODAK7_MEDIA_5TA2
 
+int kodak6_mediamax(int type);
 const char *kodak6_mediatypes(int type);
 void kodak6_dumpmediacommon(int type);
 
@@ -453,6 +593,17 @@ void kodak6_dumpmediacommon(int type);
 
 /* ********** Below are for the old S1145 (EK68xx) and S1245 only! */
 
+enum {
+	TONE_TABLE_STANDARD = 0,
+	TONE_TABLE_USER = 1,
+	TONE_TABLE_CURRENT = 2,
+};
+enum {
+	PARAM_TABLE_NONE = 0,
+	PARAM_TABLE_STANDARD = 1,
+	PARAM_TABLE_FINE = 2,
+};
+
 enum {
 	CMD_CODE_OK = 1,
 	CMD_CODE_BAD = 2,
