@@ -1,7 +1,7 @@
 /*
  *   Sony UP-D series Photo Printer CUPS backend -- libusb-1.0 version
  *
- *   (c) 2013-2019 Solomon Peachy <pizza@shaftnet.org>
+ *   (c) 2013-2020 Solomon Peachy <pizza@shaftnet.org>
  *
  *   The latest version of this program can be found at:
  *
@@ -26,16 +26,6 @@
  *
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <unistd.h>
-
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <signal.h>
-
 #define BACKEND sonyupd_backend
 
 #include "backend_common.h"
@@ -49,11 +39,12 @@ struct sony_updsts {
 	uint8_t  zero1;    /* 0x00 */
 	uint8_t  printing; /* UPD_PRINTING_* */
 	uint8_t  remain;   /* Number of remaining pages */
-	uint8_t  zero2;
+	uint8_t  sts0;     /* UPD_STS0_* */
 	uint8_t  sts1;     /* UPD_STS1_* */
-	uint8_t  sts2;     /* seconday status */
-	uint8_t  sts3;     /* tertiary status */
-	uint16_t unk;      /* seen 0x04a0 UP-CR10L, 0x04a8 on UP-DR150 */
+	uint8_t  sts2;     /* UPD_STS2_* */
+	uint8_t  sts3;     /* UPD_STS3_* */
+	uint8_t  ribbon;   /* 0x04 = R206/6x8 */
+	uint8_t  paper;    /* 0x38 = EMPTY, 0xa8 = loaded */
 	uint16_t max_cols; /* BE */
 	uint16_t max_rows; /* BE */
 	uint8_t  percent;  /* 0-99, if job is printing (UP-D89x) */
@@ -66,27 +57,35 @@ struct sony_updsts {
 #define UPD_PRINTING_O     0x20
 #define UPD_PRINTING_IDLE  0x00
 
+/* Confirmed on UP-DR200 */
+#define UPD_STS0_OK        0x00
+#define UPD_STS0_NORIBBON  0x10
+#define UPD_STS0_NOPAPER   0x20
+#define UPD_STS0_DOOROPEN  0x40
+
 #define UPD_STS1_IDLE      0x00
 #define UPD_STS1_DOOROPEN  0x08
 #define UPD_STS1_NOPAPER   0x40
 #define UPD_STS1_PRINTING  0x80
+#define UPD_STS1_PRINTING2 0xC0
+
+#define UPD_RIBBON_R206    0x04
 
 /* Private data structures */
 struct upd_printjob {
+	size_t jobsize;
+	int copies;
+
 	uint8_t *databuf;
 	int datalen;
 
-	int copies;
 	uint16_t rows;
 	uint16_t cols;
 	uint32_t imglen;
 };
 
 struct upd_ctx {
-	struct libusb_device_handle *dev;
-	uint8_t endp_up;
-	uint8_t endp_down;
-	int type;
+	struct dyesub_connection *conn;
 
 	int native_bpp;
 
@@ -95,53 +94,18 @@ struct upd_ctx {
 	struct marker marker;
 };
 
-/* Now for the code */
-static void* upd_init(void)
+static const char *upd_ribbons(int type, uint8_t code)
 {
-	struct upd_ctx *ctx = malloc(sizeof(struct upd_ctx));
-	if (!ctx) {
-		ERROR("Memory Allocation Failure!");
-		return NULL;
-	}
-	memset(ctx, 0, sizeof(struct upd_ctx));
-	return ctx;
-}
-
-static int upd_attach(void *vctx, struct libusb_device_handle *dev, int type,
-			  uint8_t endp_up, uint8_t endp_down, uint8_t jobid)
-{
-	struct upd_ctx *ctx = vctx;
-
-	UNUSED(jobid);
-
-	ctx->dev = dev;
-	ctx->endp_up = endp_up;
-	ctx->endp_down = endp_down;
-	ctx->type = type;
-
-	if (ctx->type == P_SONY_UPD895 || ctx->type == P_SONY_UPD897) {
-		ctx->marker.color = "#000000";  /* Ie black! */
-		ctx->native_bpp = 1;
-	} else {
-		ctx->marker.color = "#00FFFF#FF00FF#FFFF00";
-		ctx->native_bpp = 3;
+	if (type == P_SONY_UPD895 || type == P_SONY_UPD897) {
+		return "UP-110 Roll";
 	}
 
-	ctx->marker.name = "Unknown";
-	ctx->marker.levelmax = -1;
-	ctx->marker.levelnow = -2;
+	/* CR10L/DR200/DR150 */
+	if (code == UPD_RIBBON_R206) {
+		return "R206 (8x6)";
+	}
 
-	return CUPS_BACKEND_OK;
-}
-
-static void upd_cleanup_job(const void *vjob)
-{
-	const struct upd_printjob *job = vjob;
-
-	if (job->databuf)
-		free(job->databuf);
-
-	free((void*)job);
+	return "Unknown";
 }
 
 // UP-DR200
@@ -156,12 +120,18 @@ static void upd_cleanup_job(const void *vjob)
 // 2UPC-R155         (335)
 // 2UPC-R156         (295)
 
+// UP-CR10L
+
+// 2UPC-C13          (300)
+// 2UPC-C14          (200)
+// 2UPC-C15          (172)
+
 // print order:  ->YMCO->
 // current prints (power on)
 // total prints (lifetime)
 // f/w version
 
-static char* upd895_statuses(uint8_t code)
+static const char* upd895_statuses(uint8_t code)
 {
 	switch (code) {
 	case UPD_STS1_IDLE:
@@ -171,31 +141,50 @@ static char* upd895_statuses(uint8_t code)
 	case UPD_STS1_NOPAPER:
 		return "No paper";
 	case UPD_STS1_PRINTING:
+	case UPD_STS1_PRINTING2:
 		return "Printing";
 	default:
 		return "Unknown";
 	}
 }
 
+static const char* updr200_statuses(uint8_t code)
+{
+	switch (code) {
+	case UPD_STS0_OK:
+		return "OK";
+	case UPD_STS0_DOOROPEN:
+		return "Door open";
+	case UPD_STS0_NOPAPER:
+		return "No paper";
+	case UPD_STS0_NORIBBON:
+		return "No ribbon";
+	default:
+		return "Unknown";
+	}
+}
+
+/* Now for the code */
+
 static int sony_get_status(struct upd_ctx *ctx, struct sony_updsts *buf)
 {
 	int ret, num = 0;
 	uint8_t query[7] = { 0x1b, 0xe0, 0, 0, 0, 0x0f, 0 };
 
-	if (ctx->type == P_SONY_UPD895)
+	if (ctx->conn->type == P_SONY_UPD895)
 		query[5] = 0x0e;
 
-	if ((ret = send_data(ctx->dev, ctx->endp_down,
+	if ((ret = send_data(ctx->conn,
 			     query, sizeof(query))))
 		return CUPS_BACKEND_FAILED;
 
-	ret = read_data(ctx->dev, ctx->endp_up, (uint8_t*) buf, sizeof(*buf),
+	ret = read_data(ctx->conn, (uint8_t*) buf, sizeof(*buf),
 			&num);
 
 	if (ret < 0)
 		return CUPS_BACKEND_FAILED;
 #if 0
-	if (ctx->type == P_SONY_UPD895 && ret != 14)
+	if (ctx->conn->type == P_SONY_UPD895 && ret != 14)
 		return CUPS_BACKEND_FAILED;
 	else if (ret != 15)
 		return CUPS_BACKEND_FAILED;
@@ -207,6 +196,63 @@ static int sony_get_status(struct upd_ctx *ctx, struct sony_updsts *buf)
 	return CUPS_BACKEND_OK;
 }
 
+static void* upd_init(void)
+{
+	struct upd_ctx *ctx = malloc(sizeof(struct upd_ctx));
+	if (!ctx) {
+		ERROR("Memory Allocation Failure!\n");
+		return NULL;
+	}
+	memset(ctx, 0, sizeof(struct upd_ctx));
+	return ctx;
+}
+
+static int upd_attach(void *vctx, struct dyesub_connection *conn, uint8_t jobid)
+{
+	struct upd_ctx *ctx = vctx;
+
+	UNUSED(jobid);
+
+	ctx->conn = conn;
+
+	if (ctx->conn->type == P_SONY_UPD895 || ctx->conn->type == P_SONY_UPD897) {
+		ctx->marker.color = "#000000";  /* Ie black! */
+		ctx->native_bpp = 1;
+	} else {
+		ctx->marker.color = "#00FFFF#FF00FF#FFFF00";
+		ctx->native_bpp = 3;
+	}
+
+	if (test_mode < TEST_MODE_NOATTACH) {
+		int ret;
+		if ((ret = sony_get_status(ctx, &ctx->stsbuf))) {
+			return ret;
+		}
+	}
+
+	if (test_mode >= TEST_MODE_NOATTACH && getenv("MEDIA_CODE")) {
+		ctx->marker.numtype = atoi(getenv("MEDIA_CODE"));
+	} else {
+		ctx->marker.numtype = ctx->stsbuf.paper;
+	}
+
+	ctx->marker.name = upd_ribbons(ctx->conn->type, ctx->stsbuf.ribbon);
+	ctx->marker.levelmax = CUPS_MARKER_UNAVAILABLE;
+	ctx->marker.levelnow = CUPS_MARKER_UNKNOWN;
+
+	return CUPS_BACKEND_OK;
+}
+
+static void upd_cleanup_job(const void *vjob)
+{
+	const struct upd_printjob *job = vjob;
+
+	if (job->databuf)
+		free(job->databuf);
+
+	free((void*)job);
+}
+
 #define MAX_PRINTJOB_LEN (2048*2764*3 + 2048)
 
 static int upd_read_parse(void *vctx, const void **vjob, int data_fd, int copies) {
@@ -258,14 +304,14 @@ static int upd_read_parse(void *vctx, const void **vjob, int data_fd, int copies
 				if(dyesub_debug)
 					DEBUG("Block ID '%08x' (len %d)\n", len, 0);
 				len = 0;
-				if (ctx->type == P_SONY_UPDR150)
+				if (ctx->conn->type == P_SONY_UPDR150)
 					run = 0;
 				break;
 			case 0xfffffff7:
 				if(dyesub_debug)
 					DEBUG("Block ID '%08x' (len %d)\n", len, 0);
 				len = 0;
-				if (ctx->type == P_SONY_UPCR10)
+				if (ctx->conn->type == P_SONY_UPCR10)
 					run = 0;
 				break;
 			case 0xfffffff8: // 895
@@ -273,7 +319,7 @@ static int upd_read_parse(void *vctx, const void **vjob, int data_fd, int copies
 				if(dyesub_debug)
 					DEBUG("Block ID '%08x' (len %d)\n", len, 0);
 				len = 0;
-				if (ctx->type == P_SONY_UPD895 || ctx->type == P_SONY_UPD897)
+				if (ctx->conn->type == P_SONY_UPD895 || ctx->conn->type == P_SONY_UPD897)
 					run = 0;
 				break;
 			case 0xffffff97:
@@ -282,7 +328,7 @@ static int upd_read_parse(void *vctx, const void **vjob, int data_fd, int copies
 				len = 12;
 				break;
 			case 0xffffffef:
-				if (ctx->type == P_SONY_UPD895 || ctx->type == P_SONY_UPD897) {
+				if (ctx->conn->type == P_SONY_UPD895 || ctx->conn->type == P_SONY_UPD897) {
 					if(dyesub_debug)
 						DEBUG("Block ID '%08x' (len %d)\n", len, 0);
 					len = 0;
@@ -297,7 +343,7 @@ static int upd_read_parse(void *vctx, const void **vjob, int data_fd, int copies
 				len = 4;
 				break;
 			case 0xffffffec:
-				if (ctx->type == P_SONY_UPD897) {
+				if (ctx->conn->type == P_SONY_UPD897) {
 					if(dyesub_debug)
 						DEBUG("Block ID '%08x' (len %d)\n", len, 4);
 					len = 4;
@@ -374,9 +420,13 @@ static int upd_read_parse(void *vctx, const void **vjob, int data_fd, int copies
 
 	/* Some models specify copies in the print job */
 	if (copies_offset) {
-		uint16_t tmp = copies;
-		tmp = cpu_to_be16(copies);
-		memcpy(job->databuf + copies_offset, &tmp, sizeof(tmp));
+		uint16_t tmp;
+		memcpy(&tmp, job->databuf + copies_offset, sizeof(tmp));
+		tmp = be16_to_cpu(tmp);
+		if (tmp < copies) {  /* Use whichever one is larger */
+			tmp = cpu_to_be16(copies);
+			memcpy(job->databuf + copies_offset, &tmp, sizeof(tmp));
+		}
 		job->copies = 1;
 	}
 
@@ -422,9 +472,9 @@ static int upd_main_loop(void *vctx, const void *vjob) {
 
 top:
 	/* Send Unknown CMD.  Resets? */
-	if (ctx->type == P_SONY_UPD897) {
+	if (ctx->conn->type == P_SONY_UPD897) {
 		const uint8_t cmdbuf[7] = { 0x1b, 0x1f, 0, 0, 0, 0, 0 };
-		ret = send_data(ctx->dev, ctx->endp_down,
+		ret = send_data(ctx->conn,
 				cmdbuf, sizeof(cmdbuf));
 		if (ret)
 			return CUPS_BACKEND_FAILED;
@@ -447,25 +497,27 @@ top:
 	}
 
 	/* Check for idle */
-	if (ctx->stsbuf.sts1 != 0x00) {
-		if (ctx->stsbuf.sts1 == 0x80) {
+	if (ctx->stsbuf.sts1 != UPD_STS1_IDLE) {
+		if (ctx->stsbuf.sts1 == UPD_STS1_PRINTING) {
 			INFO("Waiting for printer idle...\n");
 			sleep(1);
 			goto top;
+		} else {
+			// XXX some sort of error?
 		}
 	}
 
 	/* Send RESET */
-	if (ctx->type != P_SONY_UPD895) {
+	if (ctx->conn->type != P_SONY_UPD895) {
 		const uint8_t rstbuf[7] = { 0x1b, 0x16, 0, 0, 0, 0, 0 };
-		ret = send_data(ctx->dev, ctx->endp_down,
+		ret = send_data(ctx->conn,
 				rstbuf, sizeof(rstbuf));
 		if (ret)
 			return CUPS_BACKEND_FAILED;
 	}
 
 #if 0 /* Unknown query */
-	if (ctx->type == P_SONY_UPD897) {
+	if (ctx->conn->type == P_SONY_UPD897) {
 		// -> 1b e6 00 00 00 08 00
 		// <- ???
 	}
@@ -480,7 +532,7 @@ top:
 
 		i += sizeof(uint32_t);
 
-		if ((ret = send_data(ctx->dev, ctx->endp_down,
+		if ((ret = send_data(ctx->conn,
 				     job->databuf + i, len)))
 			return CUPS_BACKEND_FAILED;
 
@@ -503,10 +555,14 @@ retry:
 	case UPD_STS1_IDLE:
 		goto done;
 	case UPD_STS1_PRINTING:
+	case UPD_STS1_PRINTING2:
 		break;
 	default:
-		ERROR("Printer error: %s (%02x)\n", upd895_statuses(ctx->stsbuf.sts1),
-		      ctx->stsbuf.sts1);
+		if (ctx->conn->type == P_SONY_UPD895 || ctx->conn->type == P_SONY_UPD897) {
+			ERROR("Printer error: %s (%02x)\n", upd895_statuses(ctx->stsbuf.sts1), ctx->stsbuf.sts1);
+		} else {
+			ERROR("Printer error: %s (%02x)\n", updr200_statuses(ctx->stsbuf.sts0), ctx->stsbuf.sts0);
+		}
 		return CUPS_BACKEND_STOP;
 	}
 
@@ -536,11 +592,18 @@ static int upd895_dump_status(struct upd_ctx *ctx)
 	if (ret < 0)
 		return CUPS_BACKEND_FAILED;
 
-	INFO("Printer status: %s (%02x)\n", upd895_statuses(ctx->stsbuf.sts1), ctx->stsbuf.sts1);
+	if (ctx->conn->type == P_SONY_UPD895 || ctx->conn->type == P_SONY_UPD897) {
+		INFO("Printer status: %s (%02x)\n", upd895_statuses(ctx->stsbuf.sts1), ctx->stsbuf.sts1);
+	} else {
+		INFO("Printer status: %s (%02x)\n", updr200_statuses(ctx->stsbuf.sts0), ctx->stsbuf.sts0);
+	}
+
 	if (ctx->stsbuf.printing != UPD_PRINTING_IDLE &&
 	    ctx->stsbuf.sts1 == UPD_STS1_PRINTING)
 		INFO("Remaining copies: %d\n", ctx->stsbuf.remain);
 
+	INFO("Media: %s (%02x)\n", upd_ribbons(ctx->conn->type, ctx->stsbuf.ribbon), ctx->stsbuf.ribbon);
+
 	return CUPS_BACKEND_OK;
 }
 
@@ -569,7 +632,7 @@ static int upd_cmdline_arg(void *vctx, int argc, char **argv)
 		if (j) return j;
 	}
 
-	return 0;
+	return CUPS_BACKEND_OK;
 }
 
 static int upd_query_markers(void *vctx, struct marker **markers, int *count)
@@ -583,25 +646,21 @@ static int upd_query_markers(void *vctx, struct marker **markers, int *count)
 	if (ret)
 		return CUPS_BACKEND_FAILED;
 
-	if (ctx->stsbuf.sts1 == 0x40 ||
-	    ctx->stsbuf.sts1 == 0x08) {
+	if (ctx->stsbuf.sts1 == UPD_STS1_NOPAPER ||
+	    ctx->stsbuf.sts1 == UPD_STS1_DOOROPEN) {
 		ctx->marker.levelnow = 0;
-		STATE("+media-empty\n");
 	} else {
-		ctx->marker.levelnow = -3;
-		STATE("-media-empty\n");
+		ctx->marker.levelnow = CUPS_MARKER_UNKNOWN_OK;
 	}
 
 	return CUPS_BACKEND_OK;
 }
 
 static const char *sonyupd_prefixes[] = {
-	"sonyupd",
-	"sony-updr150", "sony-updr200", "sony-upcr10l",
-	"sony-upd895", "sony-upd897", "dnp-sl10",
+	"sonyupd", /* Family Name */
+	"dnp-sl10", // Unknown if shared with CR10L
 	// Backwards compatibility
-	"sonyupdr150",
-	"sonyupdr200", "sonyupcr10",
+	"sonyupdr150", "sonyupdr200", "sonyupcr10",
 	NULL
 };
 
@@ -613,9 +672,9 @@ static const char *sonyupd_prefixes[] = {
 #define USB_PID_SONY_UPD895  0x0049
 #define USB_PID_SONY_UPD897  0x01E7
 
-struct dyesub_backend sonyupd_backend = {
+const struct dyesub_backend sonyupd_backend = {
 	.name = "Sony UP-D",
-	.version = "0.37",
+	.version = "0.43",
 	.uri_prefixes = sonyupd_prefixes,
 	.cmdline_arg = upd_cmdline_arg,
 	.cmdline_usage = upd_cmdline,
@@ -629,7 +688,7 @@ struct dyesub_backend sonyupd_backend = {
 		{ USB_VID_SONY, USB_PID_SONY_UPDR150, P_SONY_UPDR150, NULL, "sony-updr150"},
 		{ USB_VID_SONY, USB_PID_SONY_UPDR200, P_SONY_UPDR150, NULL, "sony-updr200"},
 		{ USB_VID_SONY, USB_PID_SONY_UPCR10, P_SONY_UPCR10, NULL, "sony-upcr10l"},
-		{ USB_VID_SONY, USB_PID_SONY_UPD895, P_SONY_UPD895, NULL, "sonyupd895"},
+		{ USB_VID_SONY, USB_PID_SONY_UPD895, P_SONY_UPD895, NULL, "sony-upd895"},
 		{ USB_VID_SONY, USB_PID_SONY_UPD897, P_SONY_UPD897, NULL, "sony-upd897"},
 		{ 0, 0, 0, NULL, NULL}
 	}
@@ -699,12 +758,12 @@ struct dyesub_backend sonyupd_backend = {
    SET PARAM
 
  <- 1b c0 00 NN LL 00 00    # LL is response length, NN is number.
- <- [ NN bytes]
+ <- [ LL bytes]
 
    QUERY PARAM
 
  <- 1b c1 00 NN LL 00 00    # LL is response length, NN is number.
- -> [ NN bytes ]
+ -> [ LL bytes ]
 
       PARAMS SEEN:
     03, len 5    [ 02 03 00 01 XX ]                   (UPDR200, 00 = normal, 02 is multicut/div2 print, 01 seen at end of stream too..
@@ -1003,22 +1062,46 @@ Other commands seen:
 
  <-- 1b 17 00 00 00 00 00   -- Unknown?
 
-   UP-CR10L
+ CR10L: status progression when printing:
 
-   2UPC-C13          (300)
-   2UPC-C14          (200)
-   2UPC-C15          (172)
+ 0e 00 00 00 00 00 00 00  04 a0 04 e0 07 38 00
+ 0e 00 00 01 00 80 00 01  04 a0 04 e0 07 38 64
+ 0e 00 40 01 00 80 00 01  04 a0 04 e0 07 38 64  <-- Y
+ 0e 00 80 01 00 80 00 01  04 a0 04 e0 07 38 64  <-- M
+ 0e 00 c0 01 00 80 00 01  04 a0 04 e0 07 38 64  <-- C
+ 0e 00 20 01 00 80 00 01  04 a0 04 e0 07 38 64  <-- O
+ 0e 00 20 01 00 80 00 01  04 a0 04 e0 07 38 00
+ 0e 00 00 01 00 80 00 01  04 a0 04 e0 07 38 00
+ 0e 00 00 00 00 00 00 00  04 a0 04 e0 07 38 00
 
- Status progression when printing:
+ UP-DR200 comms protocol:
 
- 0e 00 00 00 00 00 00 00 04 a0 04 e0 07 38 00
- 0e 00 00 01 00 80 00 01 04 a0 04 e0 07 38 64
- 0e 00 40 01 00 80 00 01 04 a0 04 e0 07 38 64  <-- Y
- 0e 00 80 01 00 80 00 01 04 a0 04 e0 07 38 64  <-- M
- 0e 00 c0 01 00 80 00 01 04 a0 04 e0 07 38 64  <-- C
- 0e 00 20 01 00 80 00 01 04 a0 04 e0 07 38 64  <-- O
- 0e 00 20 01 00 80 00 01 04 a0 04 e0 07 38 00
- 0e 00 00 01 00 80 00 01 04 a0 04 e0 07 38 00
- 0e 00 00 00 00 00 00 00 04 a0 04 e0 07 38 00
+ <-- 1b e0 00 00 00 0f 00
+ --> 0e 00 XX NN E0 E1 00 E3  RT PT CC CC RR RR %%
+
+  XX : Print state (eg Y/M/C/O -- see UPD_PRINTING_* )
+  E0 : Error code0 (see UPD_STS0_*)
+  E1 : Error code1 (see UPD_STS1_*)
+  E3 : Unknown (00 or 01)
+  NN : Number of remaining copies  (00 when complete/idle)
+  CC : Max Cols (0x0800 on 6" ribbon)
+  RR : Max Rows (0x0aa4 on 6x8)
+  RT : Ribbon type (04 = R206/8x6)
+  PT : Paper type  (38 = empty, a8 = 6")
+  %% : Percentage complete (0-99)
+
+ Seen on UP-DR200:
+
+ 0e 00 00 00 00 00 00 00  04 a8 08 00 0a a4 00   READY   (R206 6x8 ribbon)
+ 0e 00 00 00 40 00 00 00  04 a8 08 00 0a a4 00   DOOR OPEN
+ 0e 00 00 00 20 00 00 00  04 38 08 00 0a a4 00   LOAD PAPER
+ 0e 00 00 00 10 08 00 00  00 a8 08 00 0a a4 00   LOAD RIBBON
+
+ 0e 00 00 01 00 c0 00 01  04 a8 08 00 0a a4 00   PRINT START
+ 0e 00 40 01 00 c0 00 01  04 a8 08 00 0a a4 %%   PRINT Y
+ 0e 00 80 01 00 c0 00 01  04 a8 08 00 0a a4 %%   PRINT M
+ 0e 00 c0 01 00 c0 00 01  04 a8 08 00 0a a4 %%   PRINT C
+ 0e 00 20 01 00 c0 00 01  04 a8 08 00 0a a4 %%   PRINT OC
+ 0e 00 00 00 00 00 00 00  04 a8 08 00 0a a4 00   PRINT DONE / READY
 
 */
