@@ -1,5 +1,4 @@
 /*
- * "$Id: ui-utils.c,v 1.3 2006/05/28 16:59:04 rlk Exp $"
  *
  *   Main window code for Print plug-in for the GIMP.
  *
@@ -18,8 +17,7 @@
  *   for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -50,6 +48,8 @@ typedef void (*StpuiBasicCallback) (GObject *object,
 				    gpointer user_data);
 
 /*  local callbacks of dialog_new ()  */
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wpedantic"
 static gint
 dialog_delete_callback (GtkWidget *widget,
 			GdkEvent  *event,
@@ -71,6 +71,7 @@ dialog_delete_callback (GtkWidget *widget,
 
   return TRUE;
 }
+#pragma GCC diagnostic pop
 
 /**
  * dialog_create_action_areav:
@@ -79,6 +80,8 @@ dialog_delete_callback (GtkWidget *widget,
  *        buttons.
  *
  */
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wpedantic"
 static void
 dialog_create_action_areav (GtkDialog *dialog,
 			    va_list    args)
@@ -173,6 +176,7 @@ dialog_create_action_areav (GtkDialog *dialog,
       label = va_arg (args, gchar *);
     }
 }
+#pragma GCC diagnostic pop
 
 /**
  * dialog_new:
@@ -214,10 +218,10 @@ stpui_dialog_new (const gchar       *title,
   GtkWidget *dialog;
   va_list    args;
 
-  va_start (args, resizable);
-
   g_return_val_if_fail (title != NULL, NULL);
 
+  va_start (args, resizable);
+
   dialog = gtk_dialog_new ();
   gtk_window_set_title (GTK_WINDOW (dialog), title);
   gtk_window_set_position (GTK_WINDOW (dialog), position);
@@ -771,7 +775,7 @@ stpui_create_radio_button(radio_group_t *radio, GSList *group,
 			     radio->button, 1, FALSE);
   stpui_set_help_data(radio->button, gettext(radio->help));
   g_signal_connect(G_OBJECT(radio->button), "toggled",
-		   G_CALLBACK(callback), (gpointer) radio->value);
+		   G_CALLBACK(callback), cast_safe(radio->value));
   return group;
 }
 
