@@ -1,6 +1,5 @@
 
 /*
- * "$Id: print-lexmark.c,v 1.161 2012/01/19 13:25:51 m0m Exp $"
  *
  *   Print plug-in Lexmark driver for the GIMP.
  *
@@ -22,8 +21,7 @@
  *   for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 /*
@@ -37,7 +35,7 @@
  * lexmark_cap_t which defines positions are in 1/72 DPI. At
  * lexmark_print the unit will be changed dependent on the printer,
  * according to the value defined at lexmark_cap_t.x_raster_res and
- * lexmark_cap_t.y_raster_res. These two parameters are specifing the
+ * lexmark_cap_t.y_raster_res. These two parameters are specifying the
  * resolution used for positioning the printer head (it is not the
  * resolution used for printing!).
  */
@@ -368,7 +366,7 @@ static const float_param_t float_parameters[] =
       STP_PARAMETER_LEVEL_ADVANCED4, 0, 1, STP_CHANNEL_NONE, 1, 0
     }, 0.0, 5.0, 1.0, 1
   },
-};    
+};
 
 static const int float_parameter_count =
 sizeof(float_parameters) / sizeof(const float_param_t);
@@ -466,8 +464,8 @@ typedef lexmark_res_t lexmark_res_t_array[LEXM_RES_COUNT];
 typedef struct {
   int ncolors;
   unsigned int used_colors; /* specifies the head colors to be used (e.g. COLOR_MODE_K */
-  unsigned int pass_length; /* avaliable jets for one color */
-  int v_top_head_offset;    /* offset from top, wehere the first jet will be found */
+  unsigned int pass_length; /* available jets for one color */
+  int v_top_head_offset;    /* offset from top, where the first jet will be found */
   int h_catridge_offset;    /* horizontal offset of cartridges */
   int h_direction_offset;   /* Offset when printing in the other direction */
   const int *head_offset;   /* specifies the offset of head colors */
@@ -555,7 +553,7 @@ static const unsigned char outbufHeader_z52[LXM_Z52_HEADERSIZE]=
   0x03,0x60,                                  /* 0xd-0xe HE */
   0x04,0xe0,                                  /* 0xf-0x10  HS vertical pos */
   0x19,0x5c,                                  /* 0x11-0x12 */
-  0x0,0x0,                                    /* 0x13-0x14  VO between packges*/
+  0x0,0x0,                                    /* 0x13-0x14  VO between packages*/
   0x0,0x80,                                   /* 0x15-0x16 */
   0x0,0x0,0x0,0x0,0x1,0x2,0x0,0x0,0x0,0x0,0x0 /* 0x17-0x21 */
 };
@@ -595,21 +593,21 @@ static const lexmark_res_t_array lexmark_reslist_z52 =  /* LEXM_RES_COUNT entrie
 static const lexmark_inkname_t ink_types_z52[] =
 {
   /*   output_type   ncolors used_colors   pass_length  v_top_head_offset
-   *                                                        h_catridge_offset
+   *                                                        h_cartridge_offset
    *                                                           h_direction_offset
    *                                                               head_offset */
   { "CMYK",     N_("Four Color Standard"),
     {{ 1, COLOR_MODE_K,        208, 324, 0, 10, head_offset_cmyk },
      { 4, COLOR_MODE_CMYK,   192/3,   0, 0, 10, head_offset_cmyk }}},
   { "RGB",      N_("Three Color Composite"),
-    {{ 1, COLOR_MODE_K,        208, 324, 0, 10, head_offset_cmyk },  /* we ignor CMY, use black */
+    {{ 1, COLOR_MODE_K,        208, 324, 0, 10, head_offset_cmyk },  /* we ignore CMY, use black */
      { 4, COLOR_MODE_CMY,    192/3,   0, 0, 10, head_offset_cmy }}},
   { "PhotoCMYK", N_("Six Color Photo"),
     {{ 1, COLOR_MODE_K,      192/3,   0, 0, 10, head_offset_cCmMyk },
      { 6, COLOR_MODE_CcMcYK, 192/3,   0, 0, 10, head_offset_cCmMyk }}},
   { "PhotoCMY", N_("Five Color Photo Composite"),
     {{ 1, COLOR_MODE_K,        208, 324, 0, 10, head_offset_cCmMyk },
-     { 5, COLOR_MODE_CcMcY,  192/3,   0, 0, 10, head_offset_cCmMyk }}}, /* we ignor CMY, use black */
+     { 5, COLOR_MODE_CcMcY,  192/3,   0, 0, 10, head_offset_cCmMyk }}}, /* we ignore CMY, use black */
   { "Gray",     N_("Black"),
     {{ 1, COLOR_MODE_K,        208, 324, 0, 10, head_offset_cmyk },
      { 1, COLOR_MODE_K,        208, 324, 0, 10, head_offset_cmyk }}},
@@ -619,7 +617,7 @@ static const lexmark_inkname_t ink_types_z52[] =
 
 
 /*
- * 3200 sepecific stuff
+ * 3200 specific stuff
  */
 #define LXM3200_LEFTOFFS 6254
 #define LXM3200_RIGHTOFFS (LXM3200_LEFTOFFS-2120)
@@ -665,20 +663,20 @@ static const lexmark_res_t_array lexmark_reslist_3200 =   /* LEXM_RES_COUNT entr
 static const lexmark_inkname_t ink_types_3200[] =
 {
   /*   output_type   ncolors used_colors   pass_length  v_top_head_offset
-   *                                                        h_catridge_offset
+   *                                                        h_cartridge_offset
    *                                                           h_direction_offset
    *                                                               head_offset */
   { "CMYK",     N_("Four Color Standard"),
     {{ 1, COLOR_MODE_K,        208,  20, 0, 12, head_offset_cmyk },
      { 4, COLOR_MODE_CMYK,   192/3,   0, 0, 12, head_offset_cmyk }}},
   { "RGB",      N_("Three Color Composite"),
-    {{ 1, COLOR_MODE_K,        208,  20, 0, 12, head_offset_cmyk },  /* we ignor CMY, use black */
+    {{ 1, COLOR_MODE_K,        208,  20, 0, 12, head_offset_cmyk },  /* we ignore CMY, use black */
      { 4, COLOR_MODE_CMY,    192/3,   0, 0, 12, head_offset_cmy }}},
   { "PhotoCMYK", N_("Six Color Photo"),
     {{ 1, COLOR_MODE_K,      192/3,   0, 0, 12, head_offset_cCmMyk },
      { 6, COLOR_MODE_CcMcYK, 192/3,   0, 0, 12, head_offset_cCmMyk }}},
   { "PhotoCMY", N_("Five Color Photo Composite"),
-    {{ 1, COLOR_MODE_K,        208,  20, 0, 12, head_offset_cCmMyk }, /* we ignor CMY, use black */
+    {{ 1, COLOR_MODE_K,        208,  20, 0, 12, head_offset_cCmMyk }, /* we ignore CMY, use black */
      { 5, COLOR_MODE_CcMcY,  192/3,   0, 0, 12, head_offset_cCmMyk }}},
   { NULL, NULL }
 };
@@ -690,7 +688,7 @@ static const lexmark_inkname_t ink_types_3200[] =
 /* main structure */
 static const lexmark_cap_t lexmark_model_capabilities[] =
 {
-  /* default settings for unkown models */
+  /* default settings for unknown models */
 
   {   (Lex_model)-1, 8*72,11*72,180,180,20,20,20,20, LEXMARK_INK_K, LEXMARK_SLOT_ASF1, 0 },
 
@@ -762,7 +760,7 @@ static const lexmark_cap_t lexmark_model_capabilities[] =
     /*** printer internal parameters ***/
     0,         /* real left paper border */
     300,       /* real top paper border */
-    1200,      /* horizontal resolutio of ??? dpi for positioning */
+    1200,      /* horizontal resolution of ??? dpi for positioning */
     1200,      /* use a vertical resolution of 1200 dpi for positioning */
     &lexmark_reslist_3200,  /* resolution specific parameters of ?? */
     ink_types_3200,  /* supported inks */
@@ -810,14 +808,14 @@ static int model_to_index(int model)
 
 
 static const lexmark_cap_t *
-lexmark_get_model_capabilities(int model)
+lexmark_get_model_capabilities(const stp_vars_t *v, int model)
 {
   int i = model_to_index(model);
 
   if (i != -1) {
     return &(lexmark_model_capabilities[i]);
   }
-  stp_deprintf(STP_DBG_LEXMARK,
+  stp_dprintf(STP_DBG_LEXMARK, v,
 	       "lexmark: model %d not found in capabilities list.\n",model);
   return &(lexmark_model_capabilities[0]);
 }
@@ -950,8 +948,11 @@ get_media_type(const char *name, const lexmark_cap_t * caps)
   return NULL;
 }
 
-static int
-lexmark_source_type(const char *name, const lexmark_cap_t * caps)
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wunused-function"
+#pragma GCC diagnostic ignored "-Wpedantic"
+static inline int
+lexmark_source_type(const stp_vars_t *v, const char *name, const lexmark_cap_t * caps)
 {
   if (name)
     {
@@ -960,11 +961,12 @@ lexmark_source_type(const char *name, const lexmark_cap_t * caps)
       if (!strcmp(name,"ManualNP")) return 1;
     }
 
-  stp_deprintf(STP_DBG_LEXMARK,
+  stp_dprintf(STP_DBG_LEXMARK, v,
 	       "lexmark: Unknown source type '%s' - reverting to auto\n",name);
 
   return 4;
 }
+#pragma GCC diagnostic pop
 
 
 
@@ -972,7 +974,8 @@ lexmark_source_type(const char *name, const lexmark_cap_t * caps)
 lexmark_head_offset
 *******************************/
 static const lexmark_lineoff_t *
-lexmark_head_offset(int ydpi,                       /* i */
+lexmark_head_offset(const stp_vars_t *v,
+		    int ydpi,                       /* i */
 		    const char *ink_type,           /* i */
 		    const lexmark_cap_t * caps,     /* i */
 		    const lexmark_inkparam_t *ink_parameter, /* i */
@@ -980,7 +983,7 @@ lexmark_head_offset(int ydpi,                       /* i */
 {
   int i;
 
-  stp_deprintf(STP_DBG_LEXMARK, "  sizie %d,  size_v %d, size_v[0] %d\n", (int)sizeof(*lineoff_buffer), (int)sizeof(lineoff_buffer->v), (int)sizeof(lineoff_buffer->v[0]));
+  stp_dprintf(STP_DBG_LEXMARK, v, "  size %d,  size_v %d, size_v[0] %d\n", (int)sizeof(*lineoff_buffer), (int)sizeof(lineoff_buffer->v), (int)sizeof(lineoff_buffer->v[0]));
   memcpy(lineoff_buffer, ink_parameter->head_offset, sizeof(*lineoff_buffer));
 
   for (i=0; i < (sizeof(lineoff_buffer->v) / sizeof(lineoff_buffer->v[0])); i++) {
@@ -999,7 +1002,7 @@ lexmark_size_type
 static unsigned char
 lexmark_size_type(const stp_vars_t *v, const lexmark_cap_t * caps)
 {
-  const stp_papersize_t *pp = stp_get_papersize_by_size(stp_get_page_height(v),
+  const stp_papersize_t *pp = stpi_get_papersize_by_size(v, stp_get_page_height(v),
 							stp_get_page_width(v));
   if (pp)
     {
@@ -1040,9 +1043,9 @@ static int lexmark_get_phys_resolution_horizontal(int model)
 #endif
 
 static const lexmark_res_t
-*lexmark_get_resolution_para(int model, const char *resolution)
+*lexmark_get_resolution_para(const stp_vars_t *v, int model, const char *resolution)
 {
-  const lexmark_cap_t * caps= lexmark_get_model_capabilities(model);
+  const lexmark_cap_t * caps= lexmark_get_model_capabilities(v, model);
 
   const lexmark_res_t *res = *(caps->res_parameters); /* get the resolution specific parameters of printer */
 
@@ -1059,15 +1062,15 @@ static const lexmark_res_t
 	  res++;
 	}
     }
-  stp_deprintf(STP_DBG_LEXMARK, "lexmark_get_resolution_para: resolution not found (%s)\n", resolution);
+  stp_dprintf(STP_DBG_LEXMARK, v, "lexmark_get_resolution_para: resolution not found (%s)\n", resolution);
   return NULL;
 }
 
 
 static int
-lexmark_print_bidirectional(int model, const char *resolution)
+lexmark_print_bidirectional(const stp_vars_t *v, int model, const char *resolution)
 {
-  const lexmark_res_t *res_para = lexmark_get_resolution_para(model, resolution);
+  const lexmark_res_t *res_para = lexmark_get_resolution_para(v, model, resolution);
   return !res_para->unidirectional;
 }
 
@@ -1091,11 +1094,12 @@ lexmark_sat_adjustment(const lexmark_cap_t * caps, const stp_vars_t *v)
 
 
 static void
-lexmark_describe_resolution(const stp_vars_t *v, int *x, int *y)
+lexmark_describe_resolution(const stp_vars_t *v,
+			    stp_resolution_t *x, stp_resolution_t *y)
 {
   const char *resolution = stp_get_string_parameter(v, "Resolution");
   const lexmark_res_t *res =
-    lexmark_get_resolution_para(stp_get_model_id(v), resolution);
+    lexmark_get_resolution_para(v, stp_get_model_id(v), resolution);
 
   if (res)
     {
@@ -1124,7 +1128,15 @@ static stp_parameter_list_t
 lexmark_list_parameters(const stp_vars_t *v)
 {
   stp_parameter_list_t *ret = stp_parameter_list_create();
+  stp_parameter_list_t *tmp_list;
+
   int i;
+
+  /* Set up dithering */
+  tmp_list = stp_dither_list_parameters(v);
+  stp_parameter_list_append(ret, tmp_list);
+  stp_parameter_list_destroy(tmp_list);
+
   for (i = 0; i < the_parameter_count; i++)
     stp_parameter_list_add_param(ret, &(the_parameters[i]));
   for (i = 0; i < float_parameter_count; i++)
@@ -1137,7 +1149,7 @@ lexmark_describe_output(const stp_vars_t *v)
 {
   int printing_color = 0;
   int model = stp_get_model_id(v);
-  const lexmark_cap_t *caps = lexmark_get_model_capabilities(model);
+  const lexmark_cap_t *caps = lexmark_get_model_capabilities(v, model);
   const char *print_mode = stp_get_string_parameter(v, "PrintingMode");
   const char *ink_type = stp_get_string_parameter(v, "InkType");
   const lexmark_inkparam_t *ink_parameter;
@@ -1162,7 +1174,7 @@ lexmark_parameters(const stp_vars_t *v, const char *name,
 {
   int		i;
 
-  const lexmark_cap_t * caps= lexmark_get_model_capabilities(stp_get_model_id(v));
+  const lexmark_cap_t * caps= lexmark_get_model_capabilities(v, stp_get_model_id(v));
   description->p_type = STP_PARAMETER_TYPE_INVALID;
 
   if (name == NULL)
@@ -1190,7 +1202,9 @@ lexmark_parameters(const stp_vars_t *v, const char *name,
   {
     unsigned int height_limit, width_limit;
     unsigned int min_height_limit, min_width_limit;
-    int papersizes = stp_known_papersizes();
+    const stp_papersize_list_t *paper_sizes = stpi_get_standard_papersize_list();
+    const stp_papersize_list_item_t *ptli =
+      stpi_papersize_list_get_start(paper_sizes);
     description->bounds.str = stp_string_list_create();
 
     width_limit  = caps->max_paper_width;
@@ -1198,18 +1212,25 @@ lexmark_parameters(const stp_vars_t *v, const char *name,
     min_width_limit  = caps->min_paper_width;
     min_height_limit = caps->min_paper_height;
 
-    for (i = 0; i < papersizes; i++) {
-      const stp_papersize_t *pt = stp_get_papersize_by_index(i);
-      if (strlen(pt->name) > 0 &&
-	  pt->width <= width_limit && pt->height <= height_limit &&
-	  (pt->height >= min_height_limit || pt->height == 0) &&
-	  (pt->width >= min_width_limit || pt->width == 0))
-	{
-	  if (stp_string_list_count(description->bounds.str) == 0)
-	    description->deflt.str = pt->name;
-	  stp_string_list_add_string(description->bounds.str,
-				     pt->name, gettext(pt->text));
+    while (ptli)
+      {
+	const stp_papersize_t *pt = stpi_paperlist_item_get_data(ptli);
+
+	if (pt->paper_size_type == PAPERSIZE_TYPE_STANDARD ||
+	    pt->paper_size_type == PAPERSIZE_TYPE_ENVELOPE) {
+
+	  if (strlen(pt->name) > 0 &&
+	      pt->width <= width_limit && pt->height <= height_limit &&
+	      (pt->height >= min_height_limit || pt->height == 0) &&
+	      (pt->width >= min_width_limit || pt->width == 0))
+	    {
+	      if (stp_string_list_count(description->bounds.str) == 0)
+		description->deflt.str = pt->name;
+	      stp_string_list_add_string(description->bounds.str,
+					 pt->name, gettext(pt->text));
+	    }
 	}
+	ptli = stpi_paperlist_item_next(ptli);
     }
   }
   else if (strcmp(name, "Resolution") == 0)
@@ -1291,24 +1312,24 @@ lexmark_parameters(const stp_vars_t *v, const char *name,
 static void
 internal_imageable_area(const stp_vars_t *v,   /* I */
 			int  use_paper_margins,
-			int  *left,	/* O - Left position in points */
-			int  *right,	/* O - Right position in points */
-			int  *bottom,	/* O - Bottom position in points */
-			int  *top)	/* O - Top position in points */
+			stp_dimension_t  *left,	/* O - Left position in points */
+			stp_dimension_t  *right,	/* O - Right position in points */
+			stp_dimension_t  *bottom,	/* O - Bottom position in points */
+			stp_dimension_t  *top)	/* O - Top position in points */
 {
-  int	width, length;			/* Size of page */
-  int left_margin = 0;
-  int right_margin = 0;
-  int bottom_margin = 0;
-  int top_margin = 0;
+  stp_dimension_t	width, length;			/* Size of page */
+  stp_dimension_t left_margin = 0;
+  stp_dimension_t right_margin = 0;
+  stp_dimension_t bottom_margin = 0;
+  stp_dimension_t top_margin = 0;
   const char *media_size = stp_get_string_parameter(v, "PageSize");
   const stp_papersize_t *pt = NULL;
   const lexmark_cap_t *caps =
-    lexmark_get_model_capabilities(stp_get_model_id(v));
+    lexmark_get_model_capabilities(v, stp_get_model_id(v));
 
 
   if (media_size && use_paper_margins)
-    pt = stp_get_papersize_by_name(media_size);
+    pt = stp_describe_papersize(v, media_size);
 
   stp_default_media_size(v, &width, &length);
   if (pt)
@@ -1331,22 +1352,22 @@ internal_imageable_area(const stp_vars_t *v,   /* I */
 
 static void
 lexmark_imageable_area(const stp_vars_t *v,   /* I */
-		       int  *left,	/* O - Left position in points */
-		       int  *right,	/* O - Right position in points */
-		       int  *bottom,	/* O - Bottom position in points */
-		       int  *top)	/* O - Top position in points */
+		       stp_dimension_t  *left,	/* O - Left position in points */
+		       stp_dimension_t  *right,	/* O - Right position in points */
+		       stp_dimension_t  *bottom,	/* O - Bottom position in points */
+		       stp_dimension_t  *top)	/* O - Top position in points */
 {
   internal_imageable_area(v, 1, left, right, bottom, top);
 }
 
 static void
 lexmark_limit(const stp_vars_t *v,  		/* I */
-	      int *width,
-	      int *height,
-	      int *min_width,
-	      int *min_height)
+	      stp_dimension_t *width,
+	      stp_dimension_t *height,
+	      stp_dimension_t *min_width,
+	      stp_dimension_t *min_height)
 {
-  const lexmark_cap_t * caps= lexmark_get_model_capabilities(stp_get_model_id(v));
+  const lexmark_cap_t * caps= lexmark_get_model_capabilities(v, stp_get_model_id(v));
   *width =	caps->max_paper_width;
   *height =	caps->max_paper_height;
   *min_width =  caps->min_paper_width;
@@ -1425,7 +1446,7 @@ lexmark_init_printer(const stp_vars_t *v, const lexmark_cap_t * caps,
 			break;
 
 		default:
-			stp_erprintf("Unknown printer !! %i\n", caps->model);
+			stp_eprintf(v, "Unknown printer !! %i\n", caps->model);
 			return 0;
   }
 
@@ -1569,7 +1590,7 @@ get_double_param(stp_vars_t *v, const char *param)
 /**********************************************************
  * lexmark_print() - Print an image to a LEXMARK printer.
  **********************************************************/
-/* This method should not be printer dependent (mybe it is because of nozzle count and other things) */
+/* This method should not be printer dependent (maybe it is because of nozzle count and other things) */
 /* The method will set the printing method depending on the selected printer.
    It will define the colors to be used and the resolution.
    Additionally the pass_length will be defined.
@@ -1581,18 +1602,18 @@ lexmark_do_print(stp_vars_t *v, stp_image_t *image)
 {
   int		status = 1;
   int		y;		/* Looping vars */
-  int		xdpi, ydpi;	/* Resolution */
-  int		n;		/* Output number */
-  int page_width,	/* Width of page */
+  stp_resolution_t	xdpi, ydpi;	/* Resolution */
+  stp_dimension_t page_width,	/* Width of page */
     page_height,	/* Length of page */
     page_left,
     page_top,
     page_right,
     page_bottom,
-    page_true_height,	/* True length of page */
-    out_width,	/* Width of image on page in pixles */
+    page_true_width,	/* True length of page */
+    page_true_height;	/* True length of page */
+  int
+    out_width,	/* Width of image on page in pixels */
     out_height,	/* Length of image on page */
-    out_channels,	/* Output bytes per pixel */
     length,		/* Length of raster data in bytes*/
     buf_length,     /* Length of raster data buffer (dmt) */
     errdiv,		/* Error dividend */
@@ -1601,14 +1622,12 @@ lexmark_do_print(stp_vars_t *v, stp_image_t *image)
     errline,	/* Current raster line */
     errlast;	/* Last raster line loaded */
   unsigned      zero_mask;
-  int           image_height,
-                image_width;
+  int           image_height;
   int           use_dmt = 0;
   int pass_length=0;              /* count of inkjets for one pass */
   int add_top_offset=0;              /* additional top offset */
   int printMode = 0;
-    int source;
-  /* Lexmark do not have differnet pixel sizes. We have to correct the density according the print resolution. */
+  /* Lexmark do not have different pixel sizes. We have to correct the density according the print resolution. */
   double  densityDivisor;            /* This parameter is will adapt the density according the resolution */
   double k_lower, k_upper;
   int  physical_xdpi = 0;
@@ -1639,17 +1658,15 @@ lexmark_do_print(stp_vars_t *v, stp_image_t *image)
   const char    *print_mode = stp_get_string_parameter(v, "PrintingMode");
   int printing_color = 0;
   const char	*ink_type     = stp_get_string_parameter(v, "InkType");
-  int		top = stp_get_top(v);
-  int		left = stp_get_left(v);
+  stp_dimension_t	top = stp_get_top(v);
+  stp_dimension_t	left = stp_get_left(v);
 
-  const lexmark_cap_t * caps= lexmark_get_model_capabilities(model);
+  const lexmark_cap_t * caps= lexmark_get_model_capabilities(v, model);
   const lexmark_res_t *res_para_ptr =
-    lexmark_get_resolution_para(model, resolution);
+    lexmark_get_resolution_para(v, model, resolution);
   const paper_t *media = get_media_type(media_type,caps);
   const lexmark_inkparam_t *ink_parameter;
 
-  stp_prune_inactive_options(v);
-
 #ifdef DEBUG
   dbgfileprn = lex_open_tmp_file(); /* open file with xx */
 #endif
@@ -1672,8 +1689,6 @@ lexmark_do_print(stp_vars_t *v, stp_image_t *image)
 
   stp_image_init(image);
 
-  source= lexmark_source_type(media_source,caps);
-
   /* force grayscale if image is grayscale
    *                 or single black cartridge installed
    */
@@ -1701,7 +1716,7 @@ lexmark_do_print(stp_vars_t *v, stp_image_t *image)
    */
 
   stp_describe_resolution(v, &xdpi, &ydpi);
-  stp_dprintf(STP_DBG_LEXMARK, v, "lexmark: resolution=%dx%d\n",xdpi,ydpi);
+  stp_dprintf(STP_DBG_LEXMARK, v, "lexmark: resolution=%dx%d\n",(int)xdpi,(int)ydpi);
 
   switch (res_para_ptr->resid) {
   case DPI300:
@@ -1732,15 +1747,15 @@ lexmark_do_print(stp_vars_t *v, stp_image_t *image)
 #endif
 
   if ((printMode & COLOR_MODE_PHOTO) == COLOR_MODE_PHOTO) {
-    /* in case of photo mode we have to go a bit ligther */
+    /* in case of photo mode we have to go a bit lighter */
 densityDivisor /= 1.2;
   }
 
   nozzle_separation = ydpi / physical_ydpi;
 
   horizontal_passes = xdpi / physical_xdpi;
-  stp_dprintf(STP_DBG_LEXMARK, v, "lexmark: horizontal_passes %i, xdpi %i, physical_xdpi %i\n",
-	       horizontal_passes, xdpi, physical_xdpi);
+  stp_dprintf(STP_DBG_LEXMARK, v, "lexmark: horizontal_passes %i, xdpi %d, physical_xdpi %i\n",
+	      horizontal_passes, (int)xdpi, physical_xdpi);
 
 
 
@@ -1765,12 +1780,11 @@ densityDivisor /= 1.2;
   page_width = page_right - page_left;
   page_height = page_bottom - page_top;
 
-  stp_dprintf(STP_DBG_LEXMARK, v, "page_right %d, page_left %d, page_top %d, page_bottom %d, left %d, top %d\n",page_right, page_left, page_top, page_bottom,left, top);
+  stp_dprintf(STP_DBG_LEXMARK, v, "page_right %f, page_left %f, page_top %f, page_bottom %f, left %f, top %f\n",page_right, page_left, page_top, page_bottom,left, top);
 
   image_height = stp_image_height(image);
-  image_width = stp_image_width(image);
 
-  stp_default_media_size(v, &n, &page_true_height);
+  stp_default_media_size(v, &page_true_width, &page_true_height);
   lxm3200_linetoeject = (page_true_height * 1200) / 72;
 
 
@@ -1788,11 +1802,11 @@ densityDivisor /= 1.2;
   out_height = ydpi * out_height / 72;
 
 
-  stp_dprintf(STP_DBG_LEXMARK, v, "border: left %d, x_raster_res %d, offser_left %d\n", left, caps->x_raster_res, caps->offset_left_border);
+  stp_dprintf(STP_DBG_LEXMARK, v, "border: left %f, x_raster_res %d, offset_left %d\n", left, caps->x_raster_res, caps->offset_left_border);
 
   left = ((caps->x_raster_res * left) / 72) + caps->offset_left_border;
 
-  stp_dprintf(STP_DBG_LEXMARK, v, "border: left %d\n", left);
+  stp_dprintf(STP_DBG_LEXMARK, v, "border: left %f\n", left);
 
 
 
@@ -1879,9 +1893,9 @@ densityDivisor /= 1.2;
 	      (cols.p.Y) ? 'Y' : ' ',
 	      (cols.p.k) ? 'k' : ' ');
 
-  /* initialize soft weaveing */
+  /* initialize soft weaving */
   privdata.ink_parameter = ink_parameter;
-  privdata.bidirectional = lexmark_print_bidirectional(model, resolution);
+  privdata.bidirectional = lexmark_print_bidirectional(v, model, resolution);
   privdata.outbuf = stp_malloc((((((pass_length/8)*11))+40) * out_width)+2000);
   privdata.direction = 0;
   stp_allocate_component_data(v, "Driver", NULL, NULL, &privdata);
@@ -1890,7 +1904,7 @@ densityDivisor /= 1.2;
   stp_initialize_weave(v,
 		       pass_length, /* jets */
 		       nozzle_separation, /* separation */
-		       horizontal_passes, /* h overample */
+		       horizontal_passes, /* h oversample */
 		       res_para_ptr->vertical_passes, /* v passes */
 		       res_para_ptr->vertical_oversample, /* v oversample */
 		       ncolors, /* colors */
@@ -1900,7 +1914,7 @@ densityDivisor /= 1.2;
 		       ((top * ydpi) / 72)+(((caps->offset_top_border+add_top_offset)*ydpi)
 					    /caps->y_raster_res),
 		       (page_height * ydpi) / 72,
-		       (const int *) lexmark_head_offset(ydpi, ink_type, caps, ink_parameter, &lineoff_buffer),
+		       (const int *) lexmark_head_offset(v, ydpi, ink_type, caps, ink_parameter, &lineoff_buffer),
 		       STP_WEAVE_ZIGZAG, /* weave_strategy */
 		       flush_pass,
 		       stp_fill_uncompressed,  /* fill_start */
@@ -1926,7 +1940,7 @@ densityDivisor /= 1.2;
 		stp_get_float_parameter(v, "Density") / densityDivisor,
 		densityDivisor);
 
-  /* Lexmark do not have differnet pixel sizes. We have to correct the density according the print resolution. */
+  /* Lexmark do not have different pixel sizes. We have to correct the density according the print resolution. */
   stp_scale_float_parameter(v, "Density", 1.0 / densityDivisor);
 
 
@@ -2077,7 +2091,7 @@ densityDivisor /= 1.2;
       stp_curve_destroy(sat_adjustment);
     }
 
-  out_channels = stp_color_init(v, image, 65536);
+  (void) stp_color_init(v, image, 65536);
 
   /* calculate the memory we need for one line of the printer image (hopefully we are right) */
   stp_dprintf(STP_DBG_LEXMARK, v, "---------- buffer mem size = %d\n", (((((pass_length/8)*11)/10)+40) * out_width)+200);
@@ -2170,7 +2184,8 @@ static const stp_printfuncs_t print_lexmark_printfuncs =
   stp_verify_printer_params,
   NULL,
   NULL,
-  NULL
+  NULL,
+  stpi_standard_describe_papersize
 };
 
 
@@ -2179,11 +2194,12 @@ static const stp_printfuncs_t print_lexmark_printfuncs =
    This method is printer type dependent code.
 
    This method initializes the line to be printed. It will set
-   the printer specific initialization which has to be done bofor
+   the printer specific initialization which has to be done before
    the pixels of the image could be printed.
 */
 static unsigned char *
-lexmark_init_line(int mode, unsigned char *prnBuf,
+lexmark_init_line(const stp_vars_t *v,
+		  int mode, unsigned char *prnBuf,
 		  int pass_length,
 		  int offset,    /* offset from left in 1/"x_raster_res" DIP (printer resolution)*/
 		  int width, int direction,
@@ -2198,8 +2214,8 @@ lexmark_init_line(int mode, unsigned char *prnBuf,
   int header_size = 0;
 
 
-  /*  stp_erprintf("#### width %d, length %d, pass_length %d\n", width, length, pass_length);*/
-  /* first, we wirte the line header */
+  /*  stp_eprintf(v, "#### width %d, length %d, pass_length %d\n", width, length, pass_length);*/
+  /* first, we write the line header */
   switch(caps->model)  {
   case m_z52:
   case m_z42:
@@ -2216,7 +2232,7 @@ lexmark_init_line(int mode, unsigned char *prnBuf,
 
     /* K could only be present if black is printed only. */
     if ((mode & COLOR_MODE_K) || (mode & (COLOR_MODE_K | COLOR_MODE_LC | COLOR_MODE_LM))) {
-      stp_deprintf(STP_DBG_LEXMARK, "set  photo/black catridge \n");
+      stp_dprintf(STP_DBG_LEXMARK, v, "set  photo/black cartridge \n");
       prnBuf[LX_Z52_COLOR_MODE_POS] = LX_Z52_BLACK_PRINT;
 
       if (direction) {
@@ -2224,7 +2240,7 @@ lexmark_init_line(int mode, unsigned char *prnBuf,
 	offset += ink_parameter->h_direction_offset;
       }
     } else {
-      stp_deprintf(STP_DBG_LEXMARK, "set color cartridge \n");
+      stp_dprintf(STP_DBG_LEXMARK, v, "set color cartridge \n");
       prnBuf[LX_Z52_COLOR_MODE_POS] = LX_Z52_COLOR_PRINT;
 
       if (direction) {
@@ -2368,7 +2384,7 @@ lexmark_init_line(int mode, unsigned char *prnBuf,
       break;
 
   case m_lex7500:
-    stp_erprintf("Lexmark 7500 not supported !\n");
+    stp_eprintf(v, "Lexmark 7500 not supported !\n");
     return NULL;
     break;
   }
@@ -2386,7 +2402,7 @@ typedef struct Lexmark_head_colors {
 
 /* lexmark_write
    This method is has NO printer type dependent code.
-   This method writes a single line of the print. The line consits of "pass_length"
+   This method writes a single line of the print. The line consists of "pass_length"
    pixel lines (pixels, which could be printed with one pass by the printer.
 */
 
@@ -2405,7 +2421,7 @@ lexmark_write(const stp_vars_t *v,		/* I - Print file or command */
 	      int           length,	/* I - Length of bitmap data in bytes */
 	      int           mode,	/* I - Which color */
 	      int           ydpi,		/* I - Vertical resolution */
-	      int           width,	/* I - Printed width in pixles */
+	      int           width,	/* I - Printed width in pixels */
 	      int           offset, 	/* I - Offset from left side in lexmark_cap_t.x_raster_res DPI */
 	      int           dmt)
 {
@@ -2413,7 +2429,7 @@ lexmark_write(const stp_vars_t *v,		/* I - Print file or command */
   int clen;
   int x;  /* actual vertical position */
   int y;  /* actual horizontal position */
-  int dy; /* horiz. inkjet posintion */
+  int dy; /* horiz. inkjet position */
   int x1;
   unsigned short pixelline;  /* byte to be written */
   unsigned int valid_bytes; /* bit list which tells the present bytes */
@@ -2454,7 +2470,7 @@ lexmark_write(const stp_vars_t *v,		/* I - Print file or command */
     xIter = -1;
   }
 
-  p = lexmark_init_line(mode, prnBuf, pass_length, offset, rwidth,
+  p = lexmark_init_line(v, mode, prnBuf, pass_length, offset, rwidth,
 			direction,  /* direction */
 			ink_parameter, caps);
 
@@ -2579,7 +2595,7 @@ lexmark_write(const stp_vars_t *v,		/* I - Print file or command */
     if (anyDots) {
       anyCol = 1;
     } else {
-      /* there are no dots, make empy package */
+      /* there are no dots, make empty package */
       /*     stp_dprintf(STP_DBG_LEXMARK, v, "-- empty col %i\n", x); */
     }
   }
@@ -2636,7 +2652,7 @@ const stp_vars_t *lex_open_tmp_file() {
   const stp_vars_t *ofile;
   char tmpstr[256];
 
-      stp_erprintf(" create file !\n");
+      stp_eprintf(v, " create file !\n");
   for (i=0, sprintf(tmpstr, "/tmp/xx%d.prn", i), ofile = fopen(tmpstr, "r");
        ofile != NULL;
        i++, sprintf(tmpstr, "/tmp/xx%d.prn", i), ofile = fopen(tmpstr, "r")) {
@@ -2645,14 +2661,14 @@ const stp_vars_t *lex_open_tmp_file() {
 	fclose(ofile);
       }
   }
-      stp_erprintf("Create file %s !\n", tmpstr);
+      stp_eprintf(v, "Create file %s !\n", tmpstr);
   ofile = fopen(tmpstr, "wb");
   STPI_ASSERT(ofile);
   return ofile;
 }
 
 void lex_tmp_file_deinit(const stp_vars_t *file) {
-  stp_erprintf("Close file %lx\n", file);
+  stp_eprintf(v, "Close file %lx\n", file);
   fclose(file);
 }
 
@@ -2685,7 +2701,7 @@ flush_pass(stp_vars_t *v, int passno, int vertical_subpass)
 
   int prn_mode;
   int j; /* color counter */
-  const lexmark_cap_t * caps= lexmark_get_model_capabilities(model);
+  const lexmark_cap_t * caps= lexmark_get_model_capabilities(v, model);
   int paperShift;
   Lexmark_head_colors head_colors[3]={{0, NULL,     0,  64/2, 64},
 				      {0, NULL,  64/2, 128/2, 64},
@@ -2711,15 +2727,15 @@ flush_pass(stp_vars_t *v, int passno, int vertical_subpass)
     prn_mode = PRINT_MODE_1200;
     break;
   default:
-    stp_dprintf(STP_DBG_LEXMARK, v, "Eror: Unsupported phys resolution (%d)\n", physical_xdpi);
+    stp_dprintf(STP_DBG_LEXMARK, v, "Error: Unsupported phys resolution (%d)\n", physical_xdpi);
     return;
     break;
   }
-  /* calculate paper shift and adapt actual resoution to physical positioning resolution */
+  /* calculate paper shift and adapt actual resolution to physical positioning resolution */
   paperShift = (pass->logicalpassstart - pd->last_pass_offset) * (caps->y_raster_res/ydpi);
   for (j = 0; j < pd->ncolors; j++)
     stp_dprintf(STP_DBG_LEXMARK, v, "Color %d: active %d line %p jets %d offset %ld\n",
-		j, lineactive[0].v[j], bufs[0].v[j], linecount[0].v[j],
+		j, lineactive[0].v[j], (void *)bufs[0].v[j], linecount[0].v[j],
 		lineoffs[0].v[j]);
 
       /*** do we have to print something with the color cartridge ? ***/
@@ -2774,7 +2790,7 @@ flush_pass(stp_vars_t *v, int passno, int vertical_subpass)
 		      (lwidth+7)/8, /* length,	 I - Length of bitmap data of one line in bytes */
 		      prn_mode | COLOR_MODE_C | COLOR_MODE_Y | COLOR_MODE_M,       /* mode,	 I - Which color */
 		      ydpi,                  /* ydpi,	 I - Vertical resolution */
-		      lwidth,      /* width, 	 I - Printed width in pixles*/
+		      lwidth,      /* width, 	 I - Printed width in pixels*/
 		      hoffset+microoffset,   /* offset  I - Offset from left side in x_raster_res DPI */
 		      0                      /* dmt */);
 	if (pd->bidirectional)
@@ -2782,8 +2798,8 @@ flush_pass(stp_vars_t *v, int passno, int vertical_subpass)
       }
 
 
-      /*** do we have to print somthing with black or photo cartridge ? ***/
-      /* we print with the photo or black cartidge */
+      /*** do we have to print something with black or photo cartridge ? ***/
+      /* we print with the photo or black cartridge */
 
     if (pd->jets != 208)
       {
@@ -2871,7 +2887,7 @@ flush_pass(stp_vars_t *v, int passno, int vertical_subpass)
 		  (lwidth+7)/8, /* length,	 I - Length of bitmap data of one line in bytes */
 		  prn_mode | COLOR_MODE_LC | COLOR_MODE_LM | COLOR_MODE_K,       /* mode,	 I - Which color */
 		  ydpi,                  /* ydpi,	 I - Vertical resolution */
-		  lwidth,      /* width, 	 I - Printed width in pixles*/
+		  lwidth,      /* width, 	 I - Printed width in pixels*/
 		  hoffset+microoffset,   /* offset  I - Offset from left side in x_raster_res DPI */
 		  0                      /* dmt */);
     if (pd->bidirectional)
@@ -2888,7 +2904,7 @@ flush_pass(stp_vars_t *v, int passno, int vertical_subpass)
       linecount[0].v[j] = 0;
     }
 
-  stp_deprintf(STP_DBG_LEXMARK, "lexmark_write finished\n");
+  stp_dprintf(STP_DBG_LEXMARK, v, "lexmark_write finished\n");
 
 }
 
@@ -2916,29 +2932,29 @@ static void testprint(testdata *td)
     fscanf(td->ifile, "%[^{]{%[^\"]\"%d %d %d %d\",", dummy1, dummy2, &(td->x), &(td->y), &(td->cols), &(td->deep));
     td->cols -= 1; /* we reduce it by one because fist color will be ignored */
     td->input_line = (char *)stp_malloc(td->x+10);
-    stp_erprintf("<%s> <%s>\n", dummy1, dummy2);
-    stp_erprintf("%d %d %d %d\n", td->x, td->y, td->cols, td->deep);
+    stp_eprintf(v, "<%s> <%s>\n", dummy1, dummy2);
+    stp_eprintf(v, "%d %d %d %d\n", td->x, td->y, td->cols, td->deep);
     if (td->cols > 16) {
-      stp_erprintf("too many colors !!\n");
+      stp_eprintf(v, "too many colors !!\n");
       return;
     }
 
     /* read the colors */
     fscanf(td->ifile, "%[^\"]\"%c	c %[^\"]\",", dummy1, dummy2, dummy2); /* jump over first color */
-    for (icol=0; icol < td->cols; icol++) {  /* we ignor the first color. It is "no dot". */
+    for (icol=0; icol < td->cols; icol++) {  /* we ignore the first color. It is "no dot". */
       fscanf(td->ifile, "%[^\"]\"%c	c %[^\"]\",", dummy1, &(td->colchar[icol]), dummy2);
-      stp_erprintf("colchar %d <%c>\n", i, td->colchar[icol]);
+      stp_eprintf(v, "colchar %d <%c>\n", i, td->colchar[icol]);
     }
 
 
     if (td->cols > 5) {
       td->cols = 7;
-      for (icol=0; icol < td->cols; icol++) {  /* we ignor the first color. It is "no dot". */
+      for (icol=0; icol < td->cols; icol++) {  /* we ignore the first color. It is "no dot". */
 	linebufs.v[icol] = (char *)stp_malloc((td->x+7)/8); /* allocate the color */
       }
     } else if (td->cols > 4) {
       td->cols = 5;
-      for (icol=0; icol < td->cols; icol++) {  /* we ignor the first color. It is "no dot". */
+      for (icol=0; icol < td->cols; icol++) {  /* we ignore the first color. It is "no dot". */
 	linebufs.v[icol] = (char *)stp_malloc((td->x+7)/8); /* allocate the color */
       }
     } else {
@@ -2946,7 +2962,7 @@ static void testprint(testdata *td)
       linebufs.v[0] = (char *)stp_malloc((td->x+7)/8); /* allocate the color */
     }
   } else {
-    stp_erprintf("can't open file !\n");
+    stp_eprintf(v, "can't open file !\n");
   }
 }
 
@@ -2956,13 +2972,13 @@ static void readtestprintline(testdata *td, lexmark_linebufs_t *linebufs)
   char dummy1[256];
   int icol, ix;
 
-  stp_erprintf("start readtestprintline\n");
+  stp_eprintf(v, "start readtestprintline\n");
   for (icol=0; icol < 7; icol++) {
     if (linebufs->v[icol] != NULL) {
       memset(linebufs->v[icol], 0, (td->x+7)/8);  /* clean line */
     }
   }
-  stp_erprintf("1 readtestprintline cols %d\n", td->cols);
+  stp_eprintf(v, "1 readtestprintline cols %d\n", td->cols);
 
 
   fscanf(td->ifile, "%[^\"]\"%[^\"]\",", dummy1, td->input_line);
@@ -2981,7 +2997,7 @@ static void readtestprintline(testdata *td, lexmark_linebufs_t *linebufs)
       }
     }
   }
-  /* stp_erprintf("pixchar  <%s><%s>\n",dummy1, td->input_line);*/
+  /* stp_eprintf(v, "pixchar  <%s><%s>\n",dummy1, td->input_line);*/
 }
 
 #endif
@@ -2996,14 +3012,14 @@ static stp_family_t print_lexmark_module_data =
 static int
 print_lexmark_module_init(void)
 {
-  return stp_family_register(print_lexmark_module_data.printer_list);
+  return stpi_family_register(print_lexmark_module_data.printer_list);
 }
 
 
 static int
 print_lexmark_module_exit(void)
 {
-  return stp_family_unregister(print_lexmark_module_data.printer_list);
+  return stpi_family_unregister(print_lexmark_module_data.printer_list);
 }
 
 
@@ -3024,4 +3040,3 @@ stp_module_t stp_module_data =
     print_lexmark_module_exit,
     (void *) &print_lexmark_module_data
   };
-
